/****************************************************************************
*
* (C) 1999-2003 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module        : TCPPRIV.H
* Function      : Private information for TCP/IP C Library code
* ---------------------------------------------------------------------------

$Header: TCPPRIV.H, 4, 02.12.2003 15:38:03, Andre Pribil$

*****************************************************************************************************/

#ifndef _TCPPRIV_H__
#define _TCPPRIV_H__


/*****************************************************************************/
//API defines
/*****************************************************************************/
#define API_OPENSOCKET          0x01
#define API_CLOSESOCKET         0x02
#define API_BIND                0x03  /* assign an address to an unnamed socket,port*/
#define API_CONNECT             0x04  /* TCP client only, connect  */
#define API_RECVFROM            0x05  /* UDP recvfrom */
#define API_SENDTO              0x06  /* UDP sendto*/
#define API_HTONS               0x07
#define API_INETADDR            0x08
#define API_SLEEP               0x09
#define API_MALLOC              0x0A
#define API_FREE                0x0B
#define API_GETRCV_BYTES        0x0C
#define API_ACCEPT              0x0D  /* TCP server only  */
#define API_LISTEN              0x0E  /* TCP server only  */
#define API_SEND                0x0F  /* TCP  only  */
#define API_RECV                0x10  /* TCP  only  */
#define API_INETTOASCII         0x11
#define API_RESETCONNECTION     0x12  /* TCP  only  */
#define API_SETLINGER           0x13  /* TCP  only  */
#define API_SETREUSE            0x14  /* TCP server only */
#define API_SETIPTOS            0x15  /* set Type Of Service for a socket */
#define API_SETSOCKOPT          0x16  /* set socket options*/
#define API_GETSOCKOPT          0x17  /* get socket options*/
#define API_SETBLOCKINGMODE     0x18  /* set a socket in blocking or non blocking mode*/
#define API_REGISTERCALLBACK    0x19  /* register call back function on socket events*/
#define API_GET_SOCKETERROR     0x21  /* Get the last socket error */
#define API_GET_TCP_STATE       0x22  /* Only at @Chip-RTOS versions with SNMP support, get tcp state socket state*/
#define API_FINDALL_SOCKETS     0x23  /* Only at @Chip-RTOS versions with SNMP support, get socket information*/

#define PPPCLIENT_INSTALLED     0x40  /* PPP client API calls */
#define PPPCLIENT_OPEN          0x41
#define PPPCLIENT_CLOSE         0x42
#define PPPCLIENT_GET_STATUS    0x43
#define PPPCLIE_GET_DNSIP       0x44  /* Get DNS ip negotiated by the remote PPP server */
#define PPPCLIENT_SET_OPTIONS   0x45

#define PPPSERVER_INSTALLED     0x50  /* PPP server API calls */
#define PPPSERVER_SUSPEND       0x51
#define PPPSERVER_ACTIVATE      0x52
#define PPPSERVER_GET_STATUS    0x53
#define PPPSERVER_GET_CFG       0x54
#define PPPSERVER_SET_OPTIONS   0x55

#define API_SNMP_GET            0x60  /* SNMP API calls, only available at SNMP RTOS versions */
#define API_FTP_GET_LOGIN       0x65  /* get addresses of FTP server login counters*/
#define API_TELNET_GET_LOGIN    0x66  /* get addresses of Telnet server login counters*/
#define API_GET_TELNET_STATE    0x67  /* get state of Telnet server 1:active, 0:closed*/

#define GET_INSTALLED_SERVERS   0x70  /* Get list of installed servers and devices */
#define REINIT_ETHERNET         0x71  /* Reconfigure ethernet interface */

#define DHCP_USE                0x72  /* DHCP usage */
#define DHCP_STAT               0x73
#define TCPIP_STATISTICS        0x74

#define PING_OPEN               0x75  /* Ping API */
#define PING_CLOSE              0x76
#define PING_STATISTICS         0x77

#define GET_MEMORY_INFO         0x78  /* memory */

#define SET_SERVER_IDLE_TIMEOUT 0x79
#define IP_CALLBACK             0x7A  /* Install a users callback for filter out ip incoming packets */
#define ARP_USER_CB             0x7B  /* Install a users callback for filter out arp incoming packets */

#define ADD_DEFAULT_GATEWAY     0x80  /* Routing */
#define DEL_DEFAULT_GATEWAY     0x81
#define GET_DEFAULT_GATEWAY     0x82
#define ADD_STATIC_ROUTE        0x83
#define DEL_STATIC_ROUTE        0x84

#define GET_ARP_ENTRY_BY_IP     0x85  /* ARP cache */
#define GET_ARP_ENTRY_BY_PHYS   0x86
#define DEL_ARP_ENTRY_BY_IP     0x87
#define DEL_ARP_ENTRY_BY_PHYS   0x88
#define ADD_ARP_ENTRY           0x89
#define GET_ARPROUTE_CACHE      0x8A

#define GET_IFACE_ENTRIES       0x8D  /* Interface information */

#define ADD_IGMP_MEMBERSHIP     0x90  /* IGMP */
#define DROP_IGMP_MEMBERSHIP    0x91
#define MCASTIP_TO_MACADDR      0x92

#define DEV_OPEN_IFACE          0xA0  /* Install and open interface and device driver */
#define DEV_CLOSE_IFACE         0xA1  /* Close interface */
#define DEV_RECV_IFACE          0xA2  /* Move received data into TCPIP stack */
#define DEV_RECV_WAIT           0xA3  /* Block a task until received data event has occured */
#define DEV_NOTIFY_ISR          0xA4  /* Notify TCPIP stack from an ISR, that there are data waiting to received */
#define DEV_GET_BUF             0xA5  /* Get a buffer from TCPIP stack for fill in received data*/
#define DEV_SND_COMPLETE        0xA6  /* Inform the TCPIP stack, that data has been sent*/
#define DEV_WAIT_DHCP_COMPLETE  0xA7  /* Wait for DHCP request completion*/


/*****************************************************************************/
#endif _TCPPRIV_H__
/*****************************************************************************/
//end tcppriv.h
/***************************************************************************/
