/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     minicgi.c
* Function:   This program demonstrates the usage of the CGI interface of IPC@CHIP.
*             A CGI function is installed which, when called through a webbrowser,
*             shows some information on the browser request.
*             After about two minutes the program uninstalls the CGI function and
*             exits. A browser request will now return with a messae saying
*             "Object not found".
* Notes:      The URL of the CGI function is http://<ip-address>/minicgi
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\cgi\MiniCGI\MiniCGI.c, 2, 31.03.2004 11:15:50, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"



/******************************************************************************
* Constants
******************************************************************************/
#define PAGENAME  "minicgi"  // Name of CGI function (for browser request)
#define RUNTIME   120        // Time in seconds for which the program will run



/******************************************************************************
* cgiFunction()
*
* This function will be executed when a browser requests the page named "minicgi".
* It constructs a small HTML page containing information on the browser request
* and returns it to the webserver.
******************************************************************************/
void huge _pascal cgiFunction( rpCgiPtr CgiRequest )
{
  static char buffer[1024];  // Buffer to contain web page
  char tmpBuffer[128];       // Temporary buffer


  // Construct HTML page
  sprintf( buffer, "<html>\r\n"
                   "<head><title>MiniCGI</title></head>\r\n"
                   "<body>\r\n"
                   "<h1>MiniCGI - IPC@Chip CGI example</h1>\r\n"
                   "<p>\r\n" );

  // Print HTTP message
  sprintf( tmpBuffer, "Method: %02i<br />\r\n", CgiRequest->fHttpRequest );
  strcat( buffer, tmpBuffer );

  // Print URL
  sprintf( tmpBuffer, "Url: %s<br />\r\n", CgiRequest->fPathPtr );
  strcat( buffer, tmpBuffer );

  // Print host address
  sprintf( tmpBuffer, "Host: %s<br />\r\n", CgiRequest->fHostPtr );
  strcat( buffer, tmpBuffer);

  // Print remote address
  sprintf( tmpBuffer, "Remote IP: %d.%d.%d.%d<br />\r\n",
           (int)( ( *( CgiRequest->fRemoteIPPtr ) & 0xFF000000l ) >> 24 ),
           (int)( ( *( CgiRequest->fRemoteIPPtr ) & 0x00FF0000l ) >> 16 ),
           (int)( ( *( CgiRequest->fRemoteIPPtr ) & 0x0000FF00l ) >> 8 ),
           (int)( *( CgiRequest->fRemoteIPPtr ) & 0x000000FFl ) );
  strcat( buffer, tmpBuffer );

  // Print arguments
  if( CgiRequest->fArgumentBufferLength > 0 )
    sprintf( tmpBuffer, "Arguments: %s<br />\r\n", CgiRequest->fArgumentBufferPtr );
  else
    sprintf( tmpBuffer, "No arguments<br />\r\n" );
  strcat( buffer, tmpBuffer );

  // Print bottom of page
  strcat( buffer, "</p>\r\n"
                  "<hr size=\"0\" />\r\n"
                  "<p><small><em>&copy; 2004 by BECK IPC GmbH</em></small></p>\r\n"
                  "</body>\r\n"
                  "</html>\r\n" );

  // Give it to the webserver
  CgiRequest->fHttpResponse = CgiHttpOk;
  CgiRequest->fResponseBufferPtr = buffer;
  CgiRequest->fResponseBufferLength = strlen( buffer );
}



/******************************************************************************
* main()
*
* Installs CGI function, runs for about 2 minutes and deinstalls CGI function
* again.
******************************************************************************/
#pragma argsused
int main( int argc, char *argv[] )
{
  CGI_Entry cgiEntry;         // Holds information on the CGI function
  char *pageName = PAGENAME;  // Name of CGI function (for browser request)
  unsigned int i;             // Counter variable


  // Say hello
  printf( "\r\nMiniCGI - IPC@Chip CGI example"
           "\r\n" );


  // Install CGI function
  cgiEntry.PathPtr = pageName;        // Name of the page
  cgiEntry.method = CgiHttpGet;       // HTTP method
  cgiEntry.CgiFuncPtr = cgiFunction;  // Function called on browser request

  if( CGI_Install( &cgiEntry ) != 0 )
  {
    printf( "\r\nInstalling CGI function %s failed"
            "\r\n", cgiEntry.PathPtr );
    return 1;
  }


  // Main loop: Increment a counter
  for( i = 0; i < 120; ++i )
  {
    // Sleep a second
    RTX_Sleep_Time( 1000 );
  }


  // Remove CGI function
  if( CGI_Delete( cgiEntry.PathPtr ) != 0 )
  {

    printf( "\r\nRemoving %s failed"
            "\r\n", cgiEntry.PathPtr );
    return 1;
  }


  // Say bye
  printf( "\r\nGood bye!" );

  return 0;
}



// End of file

