/****************************************************************************
*
* (C) 2001 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-240
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module        : main.c
* Function      : This short example shows how to send a message to another
						cellphone via the EXT interface. The program will first encode
						a phonenumber and a message to the SMS PDU format. After this
						several AT-commands are used to submit the short message.



* Author        : George Parks Davie

* Date          : 11/30/01

* ---------------------------------------------------------------------------
* History       :
*
*     Date            Author  Changes
      11/30/01           gpd  special characters encoding fixed part 3
      11/27/01           gpd  special characters encoding fixed part 2
      11/26/01           gpd  special characters encoding fixed part 1
      11/13/01           gpd  program build and source cleaned up


*****************************************************************************/


// ******************************************************************* includes
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>

#define FOS_INT  0x14
#define port     0x00

// ****************************************************************** functions
int init_fossil(void);
void CleanUp(void);
void SetZero(void);
void SendSMSNow(void);
void api_sleep(int ms);
void SendString(char *SStr, int Port);
void CharToSMS(char *CharMessage, int MsgLen);
void EncodeSMS(char *PhoneNumber, char *myMsg);
char *SwitchNumbers(char *Org_Number);
unsigned char CharSubSMS(char Zeichen);

// ********************************************************** SMS PDU structure
struct SMS_PDU
{
  	unsigned char SMSC_Val;
  	unsigned char Msg_Type;
   unsigned char Service_Number[20];
  	unsigned char Msg_Ref;
  	unsigned char PN_Length;
  	unsigned char PN_Kind;
   unsigned char P_Number[20];
   unsigned char P_ID;
   unsigned char DCS;
   unsigned char TimeStamp[20];
   unsigned char Val_Per;
   unsigned char Msg_Length;
   unsigned char Message[160];
};

// *********************************************** some needed global variables
int    SMSLenCount = 0;
char   myMsg[160];
char   MsgBuffer[200];
char   PhoneNumber[20];
char   SMSmsgString[450];
struct SMS_PDU SendSMS;

int flow;
unsigned long baud;

// ********************************************************** int main fucntion
void main(int args_cnt, char *args_val[])
{
	int i = 0;
   char buffer[50];

   if (args_cnt < 5)
   {
   	printf("\nWrong usage!\nPlease use as followed: smscgi.exe <baud> <flow> <cpn> <msg>\n");
   	printf("\n<baud> = baudrate (i.e. 19200 or 57600)");
   	printf("\n<flow> = flow control (0 = no; 1 = hardware)");
   	printf("\n<cpn>  = cellphone number (i.e. 491701122334)");
   	printf("\n<msg>  = message\n\n");
      return;
   }

	SetZero();											// empty some variables

	baud = atol(args_val[1]);                 // set baud value for ext
   flow = atoi(args_val[2]);                 // set flow control for ext
	strcpy(PhoneNumber, args_val[3]);         // receiver number

   memset(buffer, 0, strlen(buffer));
   strcpy(myMsg, "");

   // The message is saved in all arguments which follows th 4th arg.
	for (i=4; i<args_cnt; i++)                // our message (max. 160 char)
   {
   	sprintf(buffer, "%s ", args_val[i]);
   	strcat(myMsg, buffer);                 // append parameters to message
   }
   myMsg[strlen(myMsg)-1] = '\0';            // clear last space in message

   if (init_fossil() == 0)                   // init fossil interface
   {
		CleanUp();                             // ...on error clean up
      printf("Error on initialization");
   	return;                                // ...and quit program
   }

   EncodeSMS(PhoneNumber, myMsg);            // encode message to PDU format
	SendSMSNow();                             // send the message
}


// ****************************************************************************
// ********************************** FUNCTIONS *******************************
// ****************************************************************************

// ***************************************** function to empty global variables
void SetZero(void)
{
   memset(PhoneNumber, 0, 20);
   memset(myMsg, 0, 162);
   memset(SMSmsgString, 0, 450);
   memset((void *)&SendSMS, 0, sizeof(struct SMS_PDU));
}

// ********************************************************** send SMS function
void SendSMSNow(void)
{
	union  REGS  inregs;
	union  REGS  outregs;
	struct SREGS segregs;

	char buffer[30];
   char ch;
   int i = 0;

   SendString("AT+CMGF=0\r", port);
	api_sleep(500);

   strcpy(buffer, "");
   sprintf(buffer, "AT+CMGS=%d\r", SMSLenCount); // our message length
   SendString(buffer, port);
	api_sleep(500);

 	SendString(SMSmsgString, port); 					 // send PDU format string now
	api_sleep(500);

   strcpy(buffer, "");
   sprintf(buffer, "%c", 26);                    // char has to be "ctrl-z"
 	SendString(buffer, port);         				 // confirm sending the message
	api_sleep(500);

	// ************************************************** some debug information
   printf("\nAT+CMGF=0");
	printf("\nAT+CMGS=%d", SMSLenCount);
   printf("\n> %s", SMSmsgString);
   printf("\nSending SMS done!");
   printf("\nNow listening on port EXT...\n\n");

   for (i=0; i<5000; i++)   // run about 5 seconds
 	{
    	// Read block of 1 char
     	inregs.h.ah = 0x18;
     	inregs.x.dx = port;
     	inregs.x.cx = 0x0001;
     	segregs.es  = FP_SEG(&ch);
     	inregs.x.di = FP_OFF(&ch);
     	int86x(FOS_INT, &inregs, &outregs, &segregs);
    	if (outregs.x.ax) printf("%c",ch);
 		api_sleep(1);
 	}
   printf("\n\n...done!\n");
}

// ************************************************* convert char message to SM
void CharToSMS(char *CharMessage, int MsgLen)
{
	int i = 0;
	int j = MsgLen;
   int k = 0;
   int n = 0;
  	unsigned char tmp_Z1 = 0;
  	unsigned char tmp_Z2 = 0;
	unsigned char tmp_Z3 = 0;

   for (i=0; i<j; i++)
   {
      if (k == 8) k = 0;
     	tmp_Z1 = (CharMessage[i] >> k);
      tmp_Z2 = (CharMessage[i+1] << (7 - k));
     	tmp_Z3 = tmp_Z1 | tmp_Z2;
		SendSMS.Message[n] = tmp_Z3;
      if ((k == 7) && (i > 1)) n--;
      n++;
      k++;
	}
   SendSMS.Message[i] = '\0';
}

// *************************************** function to substitute special chars
unsigned char CharSubSMS(char Zeichen)
{
	if (Zeichen == '') return   1;
   if (Zeichen == '$') return   2;
   if (Zeichen == '') return   3;
	if (Zeichen == '') return   4;
	if (Zeichen == '') return   5;
	if (Zeichen == '') return   6;
	if (Zeichen == '') return   7;
	if (Zeichen == '') return   8;
	if (Zeichen == '') return   9;
	if (Zeichen == '@') return   0;
	if (Zeichen == '_') return  17;
   if (Zeichen == '') return  27;
   if (Zeichen == '') return  28;
   if (Zeichen == '') return  29;
   if (Zeichen == '') return  30;
	if (Zeichen == '') return  31;
   if (Zeichen == '') return  36;
   if (Zeichen == '') return  64;
   if (Zeichen == '') return  91;
   if (Zeichen == '') return  92;
   if (Zeichen == '') return  93;
   if (Zeichen == '') return  94;
   if (Zeichen == '') return  95;
   if (Zeichen == '') return  96;
   if (Zeichen == '') return 123;
   if (Zeichen == '') return 124;
   if (Zeichen == '') return 125;
   if (Zeichen == '') return 126;
	if (Zeichen == '') return 127;
   return Zeichen;
}

// ********************************************************** encode SMS string
void EncodeSMS(char *PhoneNumber, char *myMsg)
{
   int i = 0;
   int j = 0;
   div_t xtr_l;
	char Buffer[3];

   memset(MsgBuffer, 0, 200);
	xtr_l = div(strlen(PhoneNumber), 2);
   if (xtr_l.rem != 0) strcat(PhoneNumber, "F");

 	SwitchNumbers(PhoneNumber);

	for (i=0; i<strlen(myMsg); i++)
	{
      switch(myMsg[i])
      {
      	case '':
				MsgBuffer[j] = CharSubSMS(myMsg[i]);
	         j++;
	      	MsgBuffer[j] = 'e';
            break;
			default:
      		MsgBuffer[j] = CharSubSMS(myMsg[i]);
   	}
      j++;
   }

	CharToSMS(MsgBuffer, j);
   xtr_l = div(j, 8); // calculate length of signs to ne ignored

   // Set the SMS_PDU struct fields!
   SendSMS.SMSC_Val = 0x00;           			  // we need an empty SMSC value
   SendSMS.Msg_Type = 0x11;                    // adress-type = sms send
   SendSMS.Msg_Ref = 0x00;                     // phone sets reference number
	SendSMS.PN_Length = strlen(PhoneNumber);    // phonenumber length
   SendSMS.PN_Kind = 0x91;                     // phonenumber international
	memcpy(SendSMS.P_Number, PhoneNumber, SendSMS.PN_Length); // phonenumber
   SendSMS.P_ID = 0x00;                        // protocol identifier
   SendSMS.DCS = 0x00;                         // data coding scheme = PDU
   SendSMS.Val_Per = 0xAD;                     // validation period = 7 days
   SendSMS.Msg_Length = j;							  // length of the message

   strcpy(SMSmsgString, "");
   sprintf(SMSmsgString, "%02X%02X%02X%02X%02X%s%02X%02X%02X%02X", SendSMS.SMSC_Val,
           SendSMS.Msg_Type, SendSMS.Msg_Ref, SendSMS.PN_Length, SendSMS.PN_Kind,
           SendSMS.P_Number, SendSMS.P_ID, SendSMS.DCS, SendSMS.Val_Per, SendSMS.Msg_Length);

   for (i=0; i<(SendSMS.Msg_Length - xtr_l.quot); i++) // less octects than septeps !!!
   {                                                   // subtract extra characters from message length
      sprintf(Buffer, "%02X", SendSMS.Message[i]);
   	strcat(SMSmsgString, Buffer);
   }

   SMSLenCount = 8 + (strlen(PhoneNumber) / 2) + SendSMS.Msg_Length - xtr_l.quot;

   printf("\nEncoding SMS...done!\n");
}

// ************************************************************* switch numbers
char *SwitchNumbers(char *Org_Number)
{
   int  i = 0;
   int  n = 0;
   char Buffer = 0;

   n = strlen(Org_Number);
   for (i=0; i<n; i+=2)
   {
   	Buffer = Org_Number[i];
		Org_Number[i] = Org_Number[i+1];
		Org_Number[i+1] = Buffer;
   }
   Org_Number[i] = '\0';
	return(Org_Number);
}

// ******************************************************* send string function
void SendString(char* SStr, int Port)
{
	union  REGS  inregs;
	union  REGS  outregs;
	struct SREGS segregs;

  	inregs.h.ah = 0x19;
	inregs.x.dx = Port;
	inregs.x.cx = strlen(SStr);
   segregs.es  = FP_SEG(SStr);
	inregs.x.di = FP_OFF(SStr);
   int86x(FOS_INT, &inregs, &outregs, &segregs);
}

// ********************************************************* api sleep function
void api_sleep(int ms)
{
	union  REGS  inregs;
	union  REGS  outregs;
	struct SREGS segregs;

  	inregs.x.ax = 0x0900;
  	inregs.x.bx = ms;
  	int86x(0xAC, &inregs, &outregs, &segregs);
}

// ******************************************************* clean up ext & stdio
void CleanUp(void)
{
  	union  REGS  inregs;
	union  REGS  outregs;
	struct SREGS segregs;

  	inregs.h.ah = 0x05;
  	inregs.x.dx = port;
  	int86x(FOS_INT, &inregs, &outregs, &segregs);
}

// *********************************************************** intit fossil ext
int init_fossil()
{
	union  REGS  inregs;
	union  REGS  outregs;
	struct SREGS segregs;
   unsigned int rs485pin = 0;

   unsigned long maxbaud;
   int divisor, div1, div2;
   long baud1, baud2;

   // Get timer base freq.
   inregs.h.ah = 0x8A;
   inregs.h.al = 2;
   int86x(0xA1, &inregs, &outregs, &segregs);
   maxbaud = (((unsigned long)outregs.x.dx<<16)+outregs.x.ax);

	// init fossil ext port
   inregs.h.ah = 0x04;
   inregs.x.dx = 0;
   int86x(FOS_INT, &inregs, &outregs, &segregs);
   if (outregs.x.ax != 0x1954) return 0;

   div1 = (int)(maxbaud / baud);
   div2 = div1 + 1;
   baud1 = (maxbaud / div1);
   baud2 = (maxbaud / div2);
   divisor = (baud1 - baud) < (baud - baud2) ? div1 : div2;

  	// init baudrate
   inregs.h.ah = 0x81;
   inregs.h.al = 3;
   inregs.h.bh = 0;
   inregs.h.bl = 0;
   inregs.x.cx = divisor;
   inregs.x.dx = port;
   int86x(FOS_INT, &inregs, &outregs, &segregs);

   // Select RS485 pin
   if (rs485pin != 0xFF)
   {
   	inregs.h.ah = 0x82;
      inregs.h.al = rs485pin;
    	inregs.x.dx = port;
    	int86x(FOS_INT, &inregs, &outregs, &segregs);
  	}
  	else rs485pin = 10;

	// init rs485
   inregs.h.al = 0x02;
   inregs.h.ah = 0x80;
   inregs.x.dx = port;
   int86x(FOS_INT, &inregs, &outregs, &segregs);
  	printf("\nBaud divisor: %d", divisor);

   // init flow control
  	if (flow)
  	{
   	if (flow == 1) // RTS/CTS
      	inregs.h.al = 2;
		if (flow == 2) // XON/XOFF
      	inregs.h.al = 9;

    	printf("\nUsing flow control: %s", (flow==1) ? "RTS/CTS" : "XON/XOFF");

    	inregs.h.ah = 0x0F;
    	inregs.x.dx = port;
    	int86x(FOS_INT, &inregs, &outregs, &segregs);
   }

   // purge buffer
   inregs.h.ah = 0x09;
   inregs.x.dx = port;
   int86x(FOS_INT, &inregs, &outregs, &segregs);
   inregs.h.ah = 0x0A;
   inregs.x.dx = port;
   int86x(FOS_INT, &inregs, &outregs, &segregs);
   return 1;
}
