/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     timer.c
* Function:   This program demonstrates the usage of the implemented timer
*             procedure RTOS API calls of the IPC@CHIP. The timer procedure does
*             periodic counter outport at DK40 port 0x600.
* Notes:      Declaration of timer procedures:
*             For accessing global variables at the timer procedure at execution
*             time, it is necessary to declare the timer procedure as huge. This
*             loads DS at start of the function and enables the right access to
*             global variables of your program. Your timer procedure must be as
*             short as possible without any waitings or endless loops. Avoid the
*             usage of large CLib functions like printf or something else.
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\rtos\TIMER\TIMER.C, 4, 31.03.2004 11:16:05, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <conio.h>
#include "clib.h"



/******************************************************************************
* Global variables
******************************************************************************/
void huge DK40_timer(void);  //prototype timer procedure

unsigned int timerID;  //timerID storage

unsigned char count=0;  //global count at timer procedure

TimerProc_Structure TProc ={&timerID,        /*pointer to storage the unique timerID*/
                            DK40_timer,      /*pointer to the procedure to be executed*/
                            NULL,            /*dummy pointer, not used*/
                            {'N','T','E','X'}, /*unique 4 character name*/
                            10L              /*timer execution interval*/
                            };

/******************************************************************************
* DK40_timer()
*
* Inkrements the output value on DK40's outputs.
******************************************************************************/
void huge DK40_timer(void)
{

   outportb(0x600,count++);
   if(count>0x7F)count=0;
}
/******************************************************************************
* main()
******************************************************************************/
int main(void)
{
   int key=0;

   //switch stdio to our own program
   BIOS_Set_Focus( FOCUS_APPLICATION );


   printf("\r\nRTOS API timer example\r\n");

   // init chip selects with the hardware API
   pfe_enable_pcs( 6 );

   if(RTX_Install_Timer(&TProc)!=0)
   {
      printf("\r\nError installing timer");
      RTX_Remove_Timer(timerID);
      return 1;
   }                      

   if(RTX_Start_Timer(timerID)!=0)
   {
      printf("\r\nError starting timer");
      RTX_Remove_Timer(timerID);
      return 1;
   }

   printf("\r\nDK40 Timer is running, ESC exits\r\n");

   while(key!=27)
   {
     if(kbhit())key=getch();
     RTX_Sleep_Time(100);
   }

    printf("\r\nStop and removing timer");
    RTX_Remove_Timer(timerID);
    printf("\r\nExit program\r\n");
    //switch stdio to command shell
    BIOS_Set_Focus( FOCUS_BOTH );
    return 0;
}
/*************************************************************************/
//end taskexp1.c
/*************************************************************************/
