/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     udpserv.c
* Function:   This program demonstrates the usage of the TCP/IP-API of the IPC@Chip.
*             Its an echo-server that send all data received via UDP back to its
*             sender.
* Notes:      Use this in combination with udpclie.
*
*******************************************************************************
*
* $Header: UDPSERV.C, 4, 31.03.2004 11:16:08, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"



/******************************************************************************
* Constants
******************************************************************************/
#define UDP_ECHO_SERVER_DEBUG
#define TM_PORT_ECHO                      7   /*well known echo port*/
#define TM_UDPECHOBUF_SERVER_SENDSIZE  1500   /*should be large enough*/
#define TM_UDPECHOBUF_SERVER_RECVSIZE  1500   /*should be large enough*/

/*
  this switch is only for debugging:
  comment this switch, if you haved finished your application
  because the print routines takes very much processing time
*/
//#define UDP_ECHO_SERVER_DEBUG



/******************************************************************************
* Global variables
******************************************************************************/
static unsigned long recv_errors=0L;
static unsigned long send_errors=0L;

static char sendbuf[TM_UDPECHOBUF_SERVER_SENDSIZE ];
static char recvbuf[TM_UDPECHOBUF_SERVER_RECVSIZE ];



/******************************************************************************
* main()
******************************************************************************/
int main(void)
{

   struct sockaddr_in   addr;
   struct sockaddr_in   fromaddr;

   int sd;
   int result, error;
   #ifdef UDP_ECHO_SERVER_DEBUG
   int i;
   #endif


   printf("\r\nUDPEchoServer  listening at port %d\r\n",TM_PORT_ECHO);
   printf("\r\nOpen socket");

   //API call open socket
   sd = opensocket( SOCK_DGRAM, &error );
   if( sd == API_ERROR)
   {
    printf("\r\nUDPEchoServer: Socket open failed: %d", error);
    goto UDP_SERVER_DONE;
   }

   /*********************************************************/
   //bind the socket  to well known echo port
   /*********************************************************/
   //at first, fill the socketaddr struct
   printf("\r\nBind socket");

   addr.sin_family      = AF_INET;
   addr.sin_port        =  htons( TM_PORT_ECHO );
   addr.sin_addr.s_addr =  0;

   //API-Call bind
   if( bind( sd, &addr, &error ) != 0 )
   {
     printf("\r\nUDPEchoServer: Bind failed %d",error);
     goto UDP_SERVER_DONE;
   }


   while(1)   //loop forever
   {

     /**************************************************************/
     //wait for incomming data
     /**************************************************************/
     result = recvfrom( sd, recvbuf, TM_UDPECHOBUF_SERVER_RECVSIZE, MSG_BLOCKING,
                        10000L, &fromaddr, &error );
     if( result != API_ERROR )
     {
       if(result > 0)    //we have data received
       {
         #ifdef UDP_ECHO_SERVER_DEBUG
         printf("\r\nReceived data: ");
         for(i=0;i<result;i++)
         {
           printf("%c",recvbuf[i]);

         }
         #endif

         /************************************************/
         //echo the data back
         /************************************************/
         memcpy(sendbuf,recvbuf,result);

         if( sendto( sd, sendbuf, result, 0, &fromaddr, & error ) == API_ERROR )
         {
            printf("\r\nUDPEchoServer: Senderror %d",error);
            send_errors++;
         }
      }
      else
         printf("\r\nUDPEchoServer: Received 0 Bytes\r\n");
     }
     else
     {

        #ifdef UDP_ECHO_SERVER_DEBUG
        printf("\r\nUDPEchoServer: Receive Error %d\r\n",error);
        #endif
        recv_errors++;

        //sleep a little
        api_sleep( 1000 );
     }
   }//while(1)   //loop forever

   //should not happen
UDP_SERVER_DONE:
   //close socket
   printf("\r\nUDPEchoServer: Closing socket");
   
   #ifdef UDP_ECHO_SERVER_DEBUG
   if(closesocket( sd, &error )==API_ERROR)
   {
    printf("\r\nUDPEchoServer: Socket close failed: %d\r\n",error);
   }
   #endif

   return 0;
}
/*************************************************************************/
//end udpserv.c
/*************************************************************************/
