/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     cascade.c
* Function:   Example for using the hardware API. This shows how to enable the
*             interrupt cascade mode.
*
*******************************************************************************
*
* $Header: cascade.c, 2, 31.03.2004 11:15:33, Andre Pribil$
*
******************************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "clib.h"

/******************************************************************************
* Defines
******************************************************************************/
#define PCS       6
#define ICW1      (PCS*0x100)+0 //20h or A0h on PC
#define ICW2      (PCS*0x100)+1 //21h or A1h on PC
#define ICW3      (PCS*0x100)+1 //21h or A1h on PC
#define ICW4      (PCS*0x100)+1 //21h or A1h on PC

#define OCW1      (PCS*0x100)+1 //21h or A1h on PC
#define OCW2      (PCS*0x100)+0 //20h or A0h on PC
#define OCW3      (PCS*0x100)+0 //20h or A0h on PC

// ICW1
// ----
// BIT0:   1=ICW4 is needed, 0=no ICW4 needed
// BIT1:   1=single 8259, 0=cascading 8259's
// BIT2:   1=4 byte int. vectors, 0=8 byte int. vectors
// BIT3:   1=level triggered mode, 0=edge triggered mode
// BIT4:   must be 1
// BIT5-7: must be 0 for PC systems
#define ICW1_VAL  0x15

// ICW2
// ----
// BIT0-2: 0=on 80x86 systems
// BIT3-7: Vector table address T7-T3
#define ICW2_VAL  0x80

// ICW3 for slave
// --------------
// BIT0-2: master int. req. slave is attached to
// Bit3-7: must be 0
#define ICW3_VAL  0x00

// ICW4
// ----
// BIT0:   1=80x86 mode, 0=MCS 80/85 mode
// BIT1:   1=auto EOI, 0=normal EOI
// BIT2-3: slave/master buffered mode
//         (00/01=not buffered, 10=buffered slave, 11=buffered master)
// BIT4:   1=special fully nested mode, 0=sequential
// BIT5-7: unused, set to 0
#define ICW4_VAL  0x09

#define VECT0     0x80
#define VECT1     0x81
#define VECT2     0x82
#define VECT3     0x83
#define VECT4     0x84
#define VECT5     0x85
#define VECT6     0x86
#define VECT7     0x87


/******************************************************************************
* Variables
******************************************************************************/
static volatile unsigned long counter0=0;
static volatile unsigned long counter1=0;
static volatile unsigned long counter2=0;
static volatile unsigned long counter3=0;
static volatile unsigned long counter4=0;
static volatile unsigned long counter5=0;
static volatile unsigned long counter6=0;
static volatile unsigned long counter7=0;

/******************************************************************************
* Interrupt handler's
******************************************************************************/
void interrupt vect0_int (void)
{
  counter0++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

void interrupt vect1_int (void)
{
  counter1++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

void interrupt vect2_int (void)
{
  counter2++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

void interrupt vect3_int (void)
{
  counter3++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

void interrupt vect4_int (void)
{
  counter4++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

void interrupt vect5_int (void)
{
  counter5++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

void interrupt vect6_int (void)
{
  counter6++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

void interrupt vect7_int (void)
{
  counter7++;

  // Give EOI to external PIC
  outportb(OCW2, 0x20);

  // Give EOI to internal controller
  hal_give_eoi (0);
}

/******************************************************************************
* Main
******************************************************************************/
void main (void)
{
   BIOS_Set_Focus (2);

   printf("\nIRQ counter demo on cascaded interrupt controller (8259)");
   printf("\nRemember CTS1 (COM) should not be connected, because pin is used as INTA\n");

   RTX_Sleep_Time(2000);

   // Initialize hardware
   pfe_enable_pcs (PCS);
   pfe_enable_adr (0);

   // Write Initialization Command Words (ICWs) to PIC
   outportb(ICW1, ICW1_VAL);
   outportb(ICW2, ICW2_VAL);
   outportb(ICW3, ICW3_VAL);
   outportb(ICW4, ICW4_VAL);

   // Set service routines
   setvect(VECT0, vect0_int);
   setvect(VECT1, vect1_int);
   setvect(VECT2, vect2_int);
   setvect(VECT3, vect3_int);
   setvect(VECT4, vect4_int);
   setvect(VECT5, vect5_int);
   setvect(VECT6, vect6_int);
   setvect(VECT7, vect7_int);

   // Enable INT0 cascade mode
   pfe_enable_int_cascade_mode();

   // Demask all interrupts
   outportb(OCW1, 0);


   while (1)
   {
      //clear counters
      counter0 =
      counter1 =
      counter2 =
      counter3 =
      counter4 =
      counter5 =
      counter6 =
      counter7 = 0;

      // Sleep for about 1 second
      RTX_Sleep_Time (1000);

      // Print counter values
      printf("\nIRQ0:%lu, IRQ1:%lu, IRQ2:%lu, IRQ3:%lu, IRQ4:%lu, IRQ5:%lu, IRQ6:%lu, IRQ7:%lu",
             counter0, counter1, counter2, counter3, counter4, counter5, counter6, counter7);

   }

}

