/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     fastpios.c
* Function:   This example reads and writes the PIO pins. The pins must be
*             initialized before using command shell command "PIO".
*             E.g.
*
*             A:\>PIO 13 5       # init pio 13 to output
*             A:\>FASTPIOS 13 1  # set pio 13 using this demo program
*             A:\>FASTPIOS 13    # read the current pio value
*
*******************************************************************************
*
* $Header: fastpios.c, 2, 31.03.2004 11:15:39, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "clib.h"

/******************************************************************************
* Display command line options
******************************************************************************/
void usage (void)
{
  printf("\nProgram shows and modifies PIO state");
  printf("\nUsage: fastpios <pio-no> [value],  pio-no=0..13 , value=0,1");
  printf("\nIf no value is specified the currect value will shown");
  printf("\nThe PIOs must be initialized before !! (E.g. using command"
         "\nshell command PIO)"
         "\n");
}

/******************************************************************************
* Main
******************************************************************************/
void main (int argc, char *argv[])
{
  unsigned char pio;
  unsigned char mode;
  HWAPI_VECTORS hwApi;
  unsigned wState;

  // get Hardware Api Function Pointers
  hwApi.size = 2;
  pfe_get_vectors(&hwApi);

  // set mode
  if (argc==3)
  {
     pio = atoi(argv[1]);
     if (pio>13)
     {
       usage();
       return;
     }

     mode = atoi(argv[2]);
     if ((mode!=0) && (mode!=1))
     {
       usage();
       return;
     }

     wState = hwApi.readPios();
     if (mode) wState |= (1<<pio);
     else        wState &= (~(1<<pio));
     hwApi.writePios(wState);
     printf("\nSet PIO%d %s", pio, mode ? "high" : "low");
  }
  // read state
  else if (argc==2)
  {
     pio = atoi(argv[1]);
     if (pio>13)
     {
       usage();
       return;
     }

     // read pio pins
     wState = hwApi.readPios();

     printf("\nPIO%u is %s",
            pio,
            wState&(1<<pio) ? "high" : "low");
  }
  else
  {
    usage();
    return;
  }
}
