/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     int.c
* Function:   Example of using the hardware API with external interrupts.
*
*******************************************************************************
*
* $Header: INT.C, 3, 31.03.2004 11:15:45, Andre Pribil$
*
******************************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "clib.h"

/******************************************************************************
* Variables
******************************************************************************/
volatile unsigned counter;
int rtx_isr = 0;
int task_id = -1;

/******************************************************************************
* Interrupt handler
******************************************************************************/
void huge int_handler (void)
{
  counter++;
  if (rtx_isr)
  {
     /* wake dos task (main loop) */
     RTX_Wakeup (task_id);
  }
}

/******************************************************************************
* Display command line options
******************************************************************************/
void usage (void)
{
  printf("\nProgram installs interrupt counter on external INT");
  printf("\nUsage: int <irq-no> <rtx isr>"
         "\n   irq-no=0, 2, 3, 4, 5, 6"
         "\n   rtx_isr=0, 1 (optional)");
}


/******************************************************************************
* Main
******************************************************************************/
void main (int argc, char *argv[])
{
   unsigned int  i,j;
   unsigned char irq;

   if (argc==2 || argc==3)
   {
      irq = atoi(argv[1]);
      if ((irq>6) || (irq==1))
      {
        usage();
        return;
      }
   }
   else
   {
      usage();
      return;
   }

   printf("\nInstalling counter on INT%d ...", irq);
   counter = 0;

   if (argc==3  && atoi(argv[2])==1)
   {
      rtx_isr = 1;
      task_id = RTX_Get_TaskID();
      printf("\nRTX ISR: Wake Task with ID %d", task_id);
   }

   // Enable external interrupt requests
   pfe_enable_int(irq);

   // Set service interrupt routine
   if (rtx_isr)
   {
     hal_install_rtx_isr (irq, 1, int_handler);
   }
   else
   {
     hal_install_isr (irq, 1, int_handler);
   }

   for (i=0; i<30; i++)
   {
     // wait 1 sec
     if (RTX_Sleep_Time(1000) == -5)
     {
        printf("\r\nTask was waked up by the ISR");
     }

     asm cli
     j=counter;
     counter = 0;
     asm sti
     printf("\r\nCounter on INT%d: %d",irq, j);
   }

   // Delete service interrupt routine
   hal_install_isr(irq, 0, NULL);

   printf("\r\ndone");
}

