/****************************************************************************
*
* (C) 2004 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module      :  snmphlp.h
* Function    :  Protoypes/Constants
*******************************************************************************
*
* $Header: snmphlp.h, 1, 15.03.2004 10:29:19, Markus Bartat$
*
*******************************************************************************
* THIS SOFTWARE IS PROVIDED BY BECK IPC GmbH ``AS IS'' AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGE.
* ---------------------------------------------------------------------------
*/
#ifndef    _snmphlp_h_

   #include <snmpv1.h>

   #define _snmphlp_h_

   #define MIN_NUMBER_BER_1_BYTE             0x7F
   #define OID_ADRESS_STRING_LENGTH           128



   #define ASN_BOOLEAN          (0x01)
   #define ASN_INTEGER          (0x02)
   #define ASN_BIT_STR          (0x03)
   #define ASN_OCTET_STR        (0x04)
   #define ASN_NULL             (0x05)
   #define ASN_OBJECT_ID        (0x06)
   #define ASN_SEQUENCE         (0x10)

   #define ASN_UNIVERSAL        (0x00)
   #define ASN_APPLICATION      (0x40)
   #define ASN_CONTEXT          (0x80)
   #define ASN_PRIVATE          (0xC0)

   #define ASN_PRIMITIVE        (0x00)
   #define ASN_CONSTRUCTOR      (0x20)

   #define ASN_LONG_LEN         (0x80)
   #define ASN_EXTENSION_ID     (0x1F)
   #define ASN_BIT8             (0x80)




   #define SNMP_PORT            161
   #define SNMP_TRAP_PORT       162

   /* The maximum accepted packet length */


   /* SNMP versions */
   #define SNMP_VERSION_1      0

   /* The SNMP Protocol Data Units */
   #define SNMP_GET_REQ_PDU      (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x0)
   #define SNMP_GETNEXT_REQ_PDU  (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x1)
   #define SNMP_GET_RSP_PDU      (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x2)
   #define SNMP_SET_REQ_PDU      (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x3)
   #define SNMP_TRP_REQ_PDU      (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x4)
   #define SNMP_GETBULK_REQ_PDU  (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x5)
   #define SNMP_INFORM_REQ_PDU   (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x6)
   #define SNMP_TRP2_REQ_PDU     (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x7)
   #define SNMP_REPORT_PDU       (ASN_CONTEXT | ASN_CONSTRUCTOR | 0x8)

   /* SNMP version 1 error-codes */
   #define SNMP_ERROR_NOERROR            (0x0)
   #define SNMP_ERROR_TOOBIG             (0x1)
   #define SNMP_ERROR_NOSUCHNAME         (0x2)
   #define SNMP_ERROR_BADVALUE           (0x3)
   #define SNMP_ERROR_READONLY           (0x4)
   #define SNMP_ERROR_GENERR             (0x5)

   /* SNMP version 2 error-codes */
   #define SNMP_ERROR_NOACCESS            (0x6)
   #define SNMP_ERROR_WRONGTYPE           (0x7)
   #define SNMP_ERROR_WRONGLENGTH         (0x8)
   #define SNMP_ERROR_WRONGENCODING       (0x9)
   #define SNMP_ERROR_WRONGVALUE          (0x10)
   #define SNMP_ERROR_NOCREATION          (0x11)
   #define SNMP_ERROR_INCONSISTENTVALUE   (0x12)
   #define SNMP_ERROR_RESOURCEUNAVAILABLE (0x13)
   #define SNMP_ERROR_COMMITFAILED        (0x14)
   #define SNMP_ERROR_UNDOFAILED          (0x15)
   #define SNMP_ERROR_AUTHORIZATIONERROR  (0x16)
   #define SNMP_ERROR_NOTWRITABLE         (0x17)
   #define SNMP_ERROR_INCONSISTENTNAME    (0x18)

   #define SNMP_TRAP_COLDSTART            (0x0)
   #define SNMP_TRAP_WARMSTART            (0x1)
   #define SNMP_TRAP_LINKDOWN             (0x2)
   #define SNMP_TRAP_LINKUP               (0x3)
   #define SNMP_TRAP_AUTHFAIL             (0x4)
   #define SNMP_TRAP_EGPNEIGHBORLOSS      (0x5)
   #define SNMP_TRAP_ENTERPRISESPECIFIC   (0x6)


   #define DKXX_TRAP_O_7                  (0x10)


   /* SNMP VarBinds. */
   #define SNMP_NOSUCHOBJECT    (ASN_CONTEXT | ASN_PRIMITIVE | 0x0)
   #define SNMP_NOSUCHINSTANCE  (ASN_CONTEXT | ASN_PRIMITIVE | 0x1)
   #define SNMP_ENDOFMIBVIEW    (ASN_CONTEXT | ASN_PRIMITIVE | 0x2)


   /* Object Syntax - universal types */
   #define SNMP_INTEGER     ASN_INTEGER
   #define SNMP_STRING      ASN_OCTET_STR
   #define SNMP_OBJID       ASN_OBJECT_ID
   #define SNMP_NULLOBJ     ASN_NULL
   #define SNMP_BITSTRING   ASN_BIT_STR
   #define SNMP_BITS        ASN_BIT_STR

   #define SNMP_SEQUENCE   (ASN_SEQUENCE | ASN_CONSTRUCTOR)



   /* Object Syntax - application types */
   #define SNMP_IPADDRESS   (ASN_APPLICATION | 0)
   #define SNMP_COUNTER     (ASN_APPLICATION | 1)
   #define SNMP_GAUGE       (ASN_APPLICATION | 2)
   #define SNMP_UINTEGER    (ASN_APPLICATION | 2)
   #define SNMP_TIMETICKS   (ASN_APPLICATION | 3)
   #define SNMP_OPAQUE      (ASN_APPLICATION | 4)
   #define SNMP_NSAP        (ASN_APPLICATION | 5)
   #define SNMP_COUNTER64   (ASN_APPLICATION | 6)


/*****************************************************************************************/
//Prototypes
/*****************************************************************************************/
int convert_SNMPtrap_to_BER(tSNMPTrap * ptTrapMessage, unsigned char *abBerString);

int convert_BER_to_SNMP(unsigned char * abBerMessage, tSNMPMessage* ptSNMPMessage);

int SNMP_convert_to_BER(tSNMPMessage * ptSNMPMessage, unsigned char * abBerMessage);

unsigned int get_BER_send_length(unsigned char * abBerLength, unsigned int *wIndex);

int insert_BER_string(unsigned char *pbBerMessage, char *pbString, unsigned int *pwIndex);

int insert_BER_word(unsigned char *pbBerMessage, unsigned int wValue, unsigned int * wIndex);

int insert_BER_value(unsigned char * pbDest , unsigned char *pbSource, unsigned int *pwIndex);

int convert_OID_to_string(char * pbString, unsigned char * pbOID);

unsigned char *  get_SNMP_string(unsigned char * pbMessage, unsigned int *wStringLength, unsigned int *wIndex );

int get_SNMP_integer(unsigned char * pbMessage, unsigned int * wIndex);

unsigned char get_SNMP_version(unsigned char * pbMessage, unsigned int * wIndex);

int get_OID_binding(unsigned char * pbMessage, unsigned char * pbOIDString, unsigned int * pwIndex );

int get_binding_value(unsigned char * pbMessage, unsigned char * pbValue, unsigned int *  pwIndex);

int string_to_oid(unsigned char * pbBer,char * pbOIDString);

int  convert_BER_to_string(unsigned char * pbMessage, char *sString);

int copy_SNMP_request_number(unsigned char * abSource, unsigned char * abDest, unsigned int* pwIndex);

int insert_BER_dword(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex);

unsigned long get_SNMP_dword(unsigned char *pbMessage, unsigned int* wIndex);

int insert_BER_timeticks(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex);

int insert_BER_macaddress(unsigned char * pbBerMessage, unsigned char * pbAddress, unsigned int * wIndex);

int insert_BER_ipaddress(unsigned char * pbBerMessage, unsigned char * pbAddress, unsigned int * wIndex);

int insert_BER_counter(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex);

int insert_BER_gauge(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex);

void append_ip_to_oidstring(unsigned long ip,char *destOidStr);

void append_port_to_oidstring(unsigned int port,char *destOidStr);

void ip_to_array(unsigned long ip, unsigned char * bytearray);

int asn_build_int_short(int intvalue,unsigned char * data);

char *  __itoa(unsigned int n, char *cpString);

#endif  /*  _snmphlp_h_*/
/***************************************************************************************************************/
