/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     mib.c
*             Requires SC13 @CHIP-RTOS V0.91 or SC12 @CHIP-RTOS V1.10 or higher
*
* Function:   Contains MIB2 tree data (RFC1123,not complete)
*             at struct variable tMib (see also mib.h)
*             The DK40 digital output variables (see define DK40_SNMP_DEMO)
*             are appended, only for demonstrate how to append own variables
*
*             For more information see readme.txt at the Doc directory
*
*******************************************************************************
*
* $Header: mib.c, 2, 31.03.2004 11:15:50, Andre Pribil$
*
*******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <mib.h>
#include <typedef.h>
#include <snmphlp.h>


/******************************************************************************
* Externals
******************************************************************************/
//external dynamic sub mib tables, see snmpv1.c
extern tMib iface_tble[];
extern tMib udp_tble[];
extern tMib tcp_tble[];
extern tMib at_tble[];
extern tMib ipaddr_tble[];
extern tMib ipntm_tble[];



/******************************************************************************
* Globals
******************************************************************************/

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Basic OID strings, used at tMib table below
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
char const * BaseOID[] = {
       "1.3.6.1.2.1.1." ,
       "1.3.6.1.2.1.2." ,
       "1.3.6.1.2.1.2.2.1.",
       "1.3.6.1.2.1.3.1.1.",
       "1.3.6.1.2.1.4."    ,
       "1.3.6.1.2.1.4.20." ,
       "1.3.6.1.2.1.4.21." ,
       "1.3.6.1.2.1.4.22." ,
       "1.3.6.1.2.1.4."    ,
       "1.3.6.1.2.1.5."    ,
       "1.3.6.1.2.1.6."    ,
       "1.3.6.1.2.1.6.13.1." ,
       "1.3.6.1.2.1.6."      ,
       "1.3.6.1.2.1.7."      ,
       "1.3.6.1.2.1.7.5.1."  ,
       "1.3.6.1.2.1.11."     ,
       #ifdef DKXX_SNMP_DEMO
       //DK40 digital outputs
       "1.3.6.1.2.1.80.1."  ,
       #endif
       (char *)0L
};

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   Global static MIB-II table, based on data structure tMib (defined at mib.h)
   typedef struct tMib
   {
      unsigned int  baseOIDindex;  // index of basic oid string at table above
      char          *abAddress;    // oid string, appended at the basic oid string
      unsigned char bAccess;       // access type (READ, READ_WRITE,...)
      unsigned char bSyntax;       // data type (SNMP_STRING,...)
      unsigned int  wChannel;      // index for find mib entry, enum MIB_II below
      unsigned int  tbl_idx;       // index number used for table members
      struct tMib * mibPointer;    // pointer to next entry of a dynamic sub mib table
                                   // or static main mib entry of tMIB,
                                   // if a null pointer is used, the next entry is
                                   // sucessor of the static main mib table tMIB
   }tMib;
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

tMib tMIB[] =  {

      {0 , "1.0"  ,READ       ,SNMP_STRING     ,sysDescr         ,0,(tMib *)0L},
      {0 , "2.0"  ,READ       ,SNMP_OBJID      ,sysObjectID      ,0,(tMib *)0L},
      {0 , "3.0"  ,READ       ,SNMP_TIMETICKS  ,sysUpTime        ,0,(tMib *)0L},
      {0 , "4.0"  ,READ_WRITE ,SNMP_STRING     ,sysContact       ,0,(tMib *)0L},
      {0 , "5.0"  ,READ_WRITE ,SNMP_STRING     ,sysName          ,0,(tMib *)0L},
      {0 , "6.0"  ,READ_WRITE ,SNMP_STRING     ,sysLocation      ,0,(tMib *)0L},
      {0 , "7.0"  ,READ       ,SNMP_INTEGER    ,sysService       ,0,(tMib *)0L},

      //set mib tree link to separate interface table, see build_iface_mib_table
      {1 , "1.0"  ,READ       ,SNMP_INTEGER    ,ifNumber         ,0,&iface_tble[0]},

      //Interface table, used only for building separate interface mib table
      {2 , "1."  ,READ       ,SNMP_INTEGER     ,ifIndex          ,0,(tMib *)0L},
      {2 , "2."  ,READ       ,SNMP_STRING      ,ifDescr          ,0,(tMib *)0L},
      {2 , "3."  ,READ       ,SNMP_INTEGER     ,ifType           ,0,(tMib *)0L},
      {2 , "4."  ,READ       ,SNMP_INTEGER     ,ifMtu            ,0,(tMib *)0L},
      {2 , "5."  ,READ       ,SNMP_GAUGE       ,ifSpeed          ,0,(tMib *)0L},
      {2 , "6."  ,READ       ,SNMP_STRING      ,ifPhysAddress    ,0,(tMib *)0L},
      {2 , "7."  ,READ_WRITE ,SNMP_INTEGER     ,ifAdminStatus    ,0,(tMib *)0L},
      {2 , "8."  ,READ       ,SNMP_INTEGER     ,ifOperStatus     ,0,(tMib *)0L},
      {2 , "9."  ,READ       ,SNMP_TIMETICKS   ,ifLastChange     ,0,(tMib *)0L},
      {2 , "10." ,READ       ,SNMP_COUNTER     ,ifInOctets       ,0,(tMib *)0L},
      {2 , "11." ,READ       ,SNMP_COUNTER     ,ifInUcastPkts    ,0,(tMib *)0L},
      {2 , "12." ,READ       ,SNMP_COUNTER     ,ifInNucastPkts   ,0,(tMib *)0L},
      {2 , "13." ,READ       ,SNMP_COUNTER     ,ifInDiscards     ,0,(tMib *)0L},
      {2 , "14." ,READ       ,SNMP_COUNTER     ,ifInErrors       ,0,(tMib *)0L},
      {2 , "15." ,READ       ,SNMP_COUNTER     ,ifInUnknownProtos,0,(tMib *)0L},
      {2 , "16." ,READ       ,SNMP_COUNTER     ,ifOutOctets      ,0,(tMib *)0L},
      {2 , "17." ,READ       ,SNMP_COUNTER     ,ifOutUcastPkts   ,0,(tMib *)0L},
      {2 , "18." ,READ       ,SNMP_COUNTER     ,ifOutNUcastPkts  ,0,(tMib *)0L},
      {2 , "19." ,READ       ,SNMP_COUNTER     ,ifOutDicards     ,0,(tMib *)0L},
      {2 , "20." ,READ       ,SNMP_COUNTER     ,ifOutErrors      ,0,(tMib *)0L},
      {2 , "21." ,READ       ,SNMP_GAUGE       ,ifOutLen         ,0,(tMib *)0L},
      {2 , "22." ,READ       ,SNMP_OBJID       ,ifSpecific       ,0,(tMib *)0L},

      //Address translation table, used only for building separate interface mib table
      {3 , "1."  ,READ_WRITE ,SNMP_INTEGER     ,atIfIndex        ,0,(tMib *)0L},
      {3 , "2."  ,READ_WRITE ,SNMP_STRING      ,atPhysAddress    ,0,(tMib *)0L},
      {3 , "3."  ,READ_WRITE ,SNMP_IPADDRESS   ,atNetAddress     ,0,(tMib *)0L},

      //IP statistics
      {4 , "1.0"  ,READ_WRITE ,SNMP_INTEGER    ,ipForwarding     ,0,(tMib *)0L},
      {4 , "2.0"  ,READ_WRITE ,SNMP_INTEGER    ,ipDefaultTTL     ,0,(tMib *)0L},
      {4 , "3.0"  ,READ       ,SNMP_COUNTER    ,ipInReceives     ,0,(tMib *)0L},
      {4 , "4.0"  ,READ       ,SNMP_COUNTER    ,ipInHdrErrors    ,0,(tMib *)0L},
      {4 , "5.0"  ,READ       ,SNMP_COUNTER    ,ipInAddrErrors   ,0,(tMib *)0L},
      {4 , "6.0"  ,READ       ,SNMP_COUNTER    ,ipForwDatagrams  ,0,(tMib *)0L},
      {4 , "7.0"  ,READ       ,SNMP_COUNTER    ,ipInUnknownProtos,0,(tMib *)0L},
      {4 , "8.0"  ,READ       ,SNMP_COUNTER    ,ipInDiscards     ,0,(tMib *)0L},
      {4 , "9.0"  ,READ       ,SNMP_COUNTER    ,ipInDelivers     ,0,(tMib *)0L},
      {4 , "10.0" ,READ       ,SNMP_COUNTER    ,ipOutrequests    ,0,(tMib *)0L},
      {4 , "11.0" ,READ       ,SNMP_COUNTER    ,ipOutDiscards    ,0,(tMib *)0L},
      {4 , "12.0" ,READ       ,SNMP_COUNTER    ,ipOutNoRoutes    ,0,(tMib *)0L},
      {4 , "13.0" ,READ       ,SNMP_COUNTER    ,ipreasmTimeout   ,0,(tMib *)0L},
      {4 , "14.0" ,READ       ,SNMP_COUNTER    ,ipReasmRedqs     ,0,(tMib *)0L},
      {4 , "15.0" ,READ       ,SNMP_COUNTER    ,ipReasmOKs       ,0,(tMib *)0L},
      {4 , "16.0" ,READ       ,SNMP_COUNTER    ,ipReasmFails     ,0,(tMib *)0L},
      {4 , "17.0" ,READ       ,SNMP_COUNTER    ,ipFragOKs        ,0,(tMib *)0L},
      {4 , "18.0" ,READ       ,SNMP_COUNTER    ,ipFragFails      ,0,(tMib *)0L},
      {4 , "19.0" ,READ       ,SNMP_COUNTER    ,ipFragCreates    ,0,&ipaddr_tble[0]},

      //IP address table
      {5 , "1.1"  ,READ       ,SNMP_INTEGER    ,ipAdEntAddr        ,0,(tMib *)0L},
      {5 , "1.2"  ,READ       ,SNMP_INTEGER    ,ipAdEntIfIndex     ,0,(tMib *)0L},
      {5 , "1.3"  ,READ       ,SNMP_IPADDRESS  ,ipAdEntNetMask     ,0,(tMib *)0L},
      {5 , "1.4"  ,READ       ,SNMP_INTEGER    ,ipAdEntBcastAddr   ,0,(tMib *)0L},
      {5 , "1.5"  ,READ       ,SNMP_INTEGER    ,ipAdEntReasmMaxSize,0,(tMib *)0L},

      //IP route table
      {6 , "1.1"  ,READ_WRITE ,SNMP_IPADDRESS  ,ipRouteDest       ,0,(tMib *)0L},
      {6 , "1.2"  ,READ_WRITE ,SNMP_INTEGER    ,ipRouteIfIndex    ,0,(tMib *)0L},
      {6 , "1.3"  ,READ_WRITE ,SNMP_INTEGER    ,ipRouteMetric1    ,0,(tMib *)0L},
      {6 , "1.4"  ,READ_WRITE ,SNMP_INTEGER    ,ipRouteMetric2    ,0,(tMib *)0L},
      {6 , "1.5"  ,READ_WRITE ,SNMP_INTEGER    ,ipRouteMetric3    ,0,(tMib *)0L},
      {6 , "1.6"  ,READ_WRITE ,SNMP_INTEGER    ,ipRouteMetric4    ,0,(tMib *)0L},
      {6 , "1.7"  ,READ_WRITE ,SNMP_IPADDRESS  ,ipRouteNextHop    ,0,(tMib *)0L},
      {6 , "1.8"  ,READ_WRITE ,SNMP_INTEGER    ,ipRouteType       ,0,(tMib *)0L},
      {6 , "1.9"  ,READ       ,SNMP_INTEGER    ,ipRouteProto      ,0,(tMib *)0L},
      {6 , "1.10" ,READ_WRITE ,SNMP_INTEGER    ,ipRouteAge        ,0,(tMib *)0L},
      {6 , "1.11" ,READ_WRITE ,SNMP_IPADDRESS  ,ipRouteMask       ,0,(tMib *)0L},
      {6 , "1.12" ,READ_WRITE ,SNMP_INTEGER    ,ipRouteMetric     ,0,(tMib *)0L},
      {6 , "1.13" ,READ       ,SNMP_OBJID      ,ipRouteInfo       ,0,(tMib *)0L},

      //IpNetToMedia table
      {7 , "1.1." ,READ_WRITE ,SNMP_INTEGER    ,ipNetToMediaIfIndex   ,0,(tMib *)0L},
      {7 , "1.2." ,READ_WRITE ,SNMP_STRING     ,ipNetToMediaPhyAddress,0,(tMib *)0L},
      {7 , "1.3." ,READ_WRITE ,SNMP_IPADDRESS  ,ipNetToMediaNetAddress,0,(tMib *)0L},
      {7 , "1.4." ,READ_WRITE ,SNMP_INTEGER    ,ipNetToMediaType      ,0,(tMib *)0L},

      {8 , "23.0" ,READ       ,SNMP_COUNTER    ,ipRoutingDiscards     ,0,(tMib *)0L},

      //ICMP table
      {9 , "1.0"   ,READ       ,SNMP_COUNTER   ,icmpInMsgs          ,0,(tMib *)0L},
      {9 , "2.0"   ,READ       ,SNMP_COUNTER   ,icmpInErrors        ,0,(tMib *)0L},
      {9 , "3.0"   ,READ       ,SNMP_COUNTER   ,icmpInDestUnreachs  ,0,(tMib *)0L},
      {9 , "4.0"   ,READ       ,SNMP_COUNTER   ,icmpInTimeExcds     ,0,(tMib *)0L},
      {9 , "5.0"   ,READ       ,SNMP_COUNTER   ,icmpInParmProbs     ,0,(tMib *)0L},
      {9 , "6.0"   ,READ       ,SNMP_COUNTER   ,icmpInSrcQuenchs    ,0,(tMib *)0L},
      {9 , "7.0"   ,READ       ,SNMP_COUNTER   ,icmpRedirects       ,0,(tMib *)0L},
      {9 , "8.0"   ,READ       ,SNMP_COUNTER   ,icmpInEchos         ,0,(tMib *)0L},
      {9 , "9.0"   ,READ       ,SNMP_COUNTER   ,icmpInEchoReps      ,0,(tMib *)0L},
      {9 , "10.0"  ,READ       ,SNMP_COUNTER   ,icmpInTimestamps    ,0,(tMib *)0L},
      {9 , "11.0"  ,READ       ,SNMP_COUNTER   ,icmpInTimestampReps ,0,(tMib *)0L},
      {9 , "12.0"  ,READ       ,SNMP_COUNTER   ,icmpInAddrMasks     ,0,(tMib *)0L},
      {9 , "13.0"  ,READ       ,SNMP_COUNTER   ,icmpInAddrMaskReps  ,0,(tMib *)0L},
      {9 , "14.0"  ,READ       ,SNMP_COUNTER   ,icmpOutMsgs         ,0,(tMib *)0L},
      {9 , "15.0"  ,READ       ,SNMP_COUNTER   ,icmpOutErrors       ,0,(tMib *)0L},
      {9 , "16.0"  ,READ       ,SNMP_COUNTER   ,icmpOutDestUnreachs ,0,(tMib *)0L},
      {9 , "17.0"  ,READ       ,SNMP_COUNTER   ,icmpOutTimeExcds    ,0,(tMib *)0L},
      {9 , "18.0"  ,READ       ,SNMP_COUNTER   ,icmpOutParmProbs    ,0,(tMib *)0L},
      {9 , "19.0"  ,READ       ,SNMP_COUNTER   ,icmpOutSrcQuenchs   ,0,(tMib *)0L},
      {9 , "20.0"  ,READ       ,SNMP_COUNTER   ,icmpOutRedirects    ,0,(tMib *)0L},
      {9 , "21.0"  ,READ       ,SNMP_COUNTER   ,icmpOutEchos        ,0,(tMib *)0L},
      {9 , "22.0"  ,READ       ,SNMP_COUNTER   ,icmpOutEchoReps     ,0,(tMib *)0L},
      {9 , "23.0"  ,READ       ,SNMP_COUNTER   ,icmpOutTimestamps   ,0,(tMib *)0L},
      {9 , "24.0"  ,READ       ,SNMP_COUNTER   ,icmpOutTimestampReps,0,(tMib *)0L},
      {9 , "25.0"  ,READ       ,SNMP_COUNTER   ,icmpOutAddrMasks    ,0,(tMib *)0L},
      {9 , "26.0"  ,READ       ,SNMP_COUNTER   ,icmpOutAddrMaskReps ,0,(tMib *)0L},

      //TCP  statistics
      {10 , "1.0"  ,READ       ,SNMP_INTEGER   ,tcpRtoAlgorithm  ,0,(tMib *)0L},
      {10 , "2.0"  ,READ       ,SNMP_INTEGER   ,tcpRtoMin        ,0,(tMib *)0L},
      {10 , "3.0"  ,READ       ,SNMP_INTEGER   ,tcpRtoMax        ,0,(tMib *)0L},
      {10 , "4.0"  ,READ       ,SNMP_INTEGER   ,tcpMaxConn       ,0,(tMib *)0L},
      {10 , "5.0"  ,READ       ,SNMP_COUNTER   ,tcpActiveOpens   ,0,(tMib *)0L},
      {10 , "6.0"  ,READ       ,SNMP_COUNTER   ,tcpPassivOpens   ,0,(tMib *)0L},
      {10 , "7.0"  ,READ       ,SNMP_COUNTER   ,tcpAttemptFails  ,0,(tMib *)0L},
      {10 , "8.0"  ,READ       ,SNMP_COUNTER   ,tcpEstabResets   ,0,(tMib *)0L},
      {10 , "9.0"  ,READ       ,SNMP_COUNTER   ,tcpCurrEstab     ,0,(tMib *)0L},
      {10 , "10.0" ,READ       ,SNMP_COUNTER   ,tcpInSegs        ,0,(tMib *)0L},
      {10 , "11.0" ,READ       ,SNMP_COUNTER   ,tcpOutSegs       ,0,(tMib *)0L},
      {10 , "12.0" ,READ       ,SNMP_COUNTER   ,tcpRetransSegs   ,0,&tcp_tble[0]},

      //TCP connection table
      {11 , "1"    ,READ_WRITE ,SNMP_INTEGER   ,tcpConnState       ,0,(tMib *)0L},
      {11 , "2"    ,READ       ,SNMP_STRING    ,tcpConnLocalAddress,0,(tMib *)0L},
      {11 , "3"    ,READ       ,SNMP_INTEGER   ,tcpConnLocalPort   ,0,(tMib *)0L},
      {11 , "4"    ,READ       ,SNMP_IPADDRESS ,tcpConnRemAddress  ,0,(tMib *)0L},
      {11 , "5"    ,READ       ,SNMP_INTEGER   ,tcpConnRemPort     ,0,(tMib *)0L},

      //TCP  error statistics
      {12 , "14.0" ,READ       ,SNMP_COUNTER   ,tcpInErrs         ,0,(tMib *)0L},
      {12 , "15.0" ,READ       ,SNMP_COUNTER   ,tcpOutRsts        ,0,(tMib *)0L},

      //UDP
      {13 , "1.0"  ,READ       ,SNMP_COUNTER   ,udpInDatagrams    ,0,(tMib *)0L},
      {13 , "2.0"  ,READ       ,SNMP_COUNTER   ,udpNoPorts        ,0,(tMib *)0L},
      {13 , "3.0"  ,READ       ,SNMP_COUNTER   ,udpInErrors       ,0,(tMib *)0L},
      {13 , "4.0"  ,READ       ,SNMP_COUNTER   ,udpOutDatagrams   ,0,&udp_tble[0]},

      //UDP listener table
      {14 , "1"    ,READ       ,SNMP_COUNTER   ,udpLocalAddress   ,0,(tMib *)0L},
      {14 , "2"    ,READ       ,SNMP_COUNTER   ,udpLocalPort      ,0,(tMib *)0L},

      //SNMP vars
      {15 , "1.0"  ,READ       ,SNMP_COUNTER   ,snmpInPkts             ,0,(tMib *)0L},
      {15 , "2.0"  ,READ       ,SNMP_COUNTER   ,snmpOutPkts            ,0,(tMib *)0L},
      {15 , "3.0"  ,READ       ,SNMP_COUNTER   ,snmpInBadVersions      ,0,(tMib *)0L},
      {15 , "4.0"  ,READ       ,SNMP_COUNTER   ,snmpInBadCommunityNames,0,(tMib *)0L},
      {15 , "5.0"  ,READ       ,SNMP_COUNTER   ,snmpInBadCommunityUses ,0,(tMib *)0L},
      {15 , "6.0"  ,READ       ,SNMP_COUNTER   ,snmpInASNParseErrs     ,0,(tMib *)0L},
      {15 , "8.0"  ,READ       ,SNMP_COUNTER   ,snmpInTooBigs          ,0,(tMib *)0L},
      {15 , "9.0"  ,READ       ,SNMP_COUNTER   ,snmpInNoSuchNames      ,0,(tMib *)0L},
      {15 , "10.0" ,READ       ,SNMP_COUNTER   ,snmpInBadValues        ,0,(tMib *)0L},
      {15 , "11.0" ,READ       ,SNMP_COUNTER   ,snmpInReadOnlys        ,0,(tMib *)0L},
      {15 , "12.0" ,READ       ,SNMP_COUNTER   ,snmpIngGenErrs         ,0,(tMib *)0L},
      {15 , "13.0" ,READ       ,SNMP_COUNTER   ,snmpInTotalReqVars     ,0,(tMib *)0L},
      {15 , "14.0" ,READ       ,SNMP_COUNTER   ,snmpInTotalSetVars     ,0,(tMib *)0L},
      {15 , "15.0" ,READ       ,SNMP_COUNTER   ,snmpInGetRequests      ,0,(tMib *)0L},
      {15 , "16.0" ,READ       ,SNMP_COUNTER   ,snmpInGetNexts         ,0,(tMib *)0L},
      {15 , "17.0" ,READ       ,SNMP_COUNTER   ,snmpInSetReqeusts      ,0,(tMib *)0L},
      {15 , "18.0" ,READ       ,SNMP_COUNTER   ,snmpInGetResponses     ,0,(tMib *)0L},
      {15 , "19.0" ,READ       ,SNMP_COUNTER   ,snmpInTraps            ,0,(tMib *)0L},
      {15 , "20.0" ,READ       ,SNMP_COUNTER   ,snmpOutTooBigs         ,0,(tMib *)0L},
      {15 , "21.0" ,READ       ,SNMP_COUNTER   ,snmpOutNoSuchNames     ,0,(tMib *)0L},
      {15 , "22.0" ,READ       ,SNMP_COUNTER   ,snmpOutBadValues       ,0,(tMib *)0L},
      {15 , "24.0" ,READ       ,SNMP_COUNTER   ,snmpOutGenErrs         ,0,(tMib *)0L},
      {15 , "25.0" ,READ       ,SNMP_COUNTER   ,snmpOutGetRequests     ,0,(tMib *)0L},
      {15 , "26.0" ,READ       ,SNMP_COUNTER   ,snmpOutGetNexts        ,0,(tMib *)0L},
      {15 , "27.0" ,READ       ,SNMP_COUNTER   ,snmpOutSetRequests     ,0,(tMib *)0L},
      {15 , "28.0" ,READ       ,SNMP_COUNTER   ,snmpOutGetResponses    ,0,(tMib *)0L},
      {15 , "29.0" ,READ       ,SNMP_COUNTER   ,snmpOutTraps           ,0,(tMib *)0L},
      {15 , "30.0" ,READ_WRITE ,SNMP_INTEGER   ,snmpEnableAuthTraps    ,0,(tMib *)0L},

      /********************************************************************************/
      #ifdef DKXX_SNMP_DEMO
      //DK40/DK50 digital outputs
      {16 , "1.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut0  ,0,(tMib *)0L},
      {16 , "2.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut1  ,0,(tMib *)0L},
      {16 , "3.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut2  ,0,(tMib *)0L},
      {16 , "4.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut3  ,0,(tMib *)0L},
      {16 , "5.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut4  ,0,(tMib *)0L},
      {16 , "6.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut5  ,0,(tMib *)0L},
      {16 , "7.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut6  ,0,(tMib *)0L},
      {16 , "8.0"  ,READ_WRITE   ,SNMP_INTEGER  ,dkxxOut7  ,0,(tMib *)0L},
      #endif
      /********************************************************************************/
      {0,(char  *)0L,0,0,0}   //mark end of mib with a null pointer OID
};
/**************************************************************************************/
//end mib.c
