/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     snmphlp.c
*
* Function:   Helper function for Encoding/Decoding SNMP/BER messages
*
*             For more information see readme.txt at the Doc directory
*
*******************************************************************************
*
* $Header: snmphlp.c, 2, 31.03.2004 11:16:00, Andre Pribil$
*
******************************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include <STDIO.H>
#include <DOS.H>
#include <STRING.H>
#include <STDLIB.H>

#include <err.h>
#include <typedef.h>
#include <snmphlp.h>

#include <bios_api.h>

/******************************************************************************
* Defines
******************************************************************************/
#define LENGTH_SNMP_VERSION    3


/******************************************************************************
* Prototypes
******************************************************************************/
static unsigned char near number_bytes_length(unsigned int);
static int near insert_BER_oid(unsigned char *pbBerMessage,
                               unsigned char * pbAddress, unsigned int *pwIndex);
static unsigned int near get_BER_length_with_length(unsigned char * pbMessage);
static unsigned int near get_length_length(unsigned char * pbMessage);
static int near insert_BER_length(unsigned char *pbBerMessage,
                                  unsigned int wLength,unsigned int *pbIndex);


/********************************************************************************/
// Convert BER (Basic Encoding Rules) endcoded message to  SNMP
/********************************************************************************/
int convert_BER_to_SNMP(unsigned char * abBerMessage, tSNMPMessage* ptSNMPMessage)
{
   int iReturn = NO_ERR;
   unsigned int wIndex;
   unsigned int wLength;
   unsigned char *pbString;
   unsigned int wStrLength;


   wIndex = 0;
   if (abBerMessage[wIndex++] ==  SNMP_SEQUENCE )
   {
       wLength = get_BER_send_length(&abBerMessage[wIndex],&wIndex );
       //Get Version
       ptSNMPMessage -> tSNMPHeadData.bVersion = get_SNMP_version(&abBerMessage[wIndex], &wIndex);
       // Get SNMP Community String
       pbString = get_SNMP_string(&abBerMessage[wIndex],&wStrLength,&wIndex);
       if (pbString != NULL)
       {
           if (wStrLength < 80)
           {
               memcpy(ptSNMPMessage -> tSNMPHeadData.bCommunityString,pbString,wStrLength);
               ptSNMPMessage -> tSNMPHeadData.bCommunityString[wStrLength] = NULL;
               // Get command
               ptSNMPMessage -> tPUDDataHead.bType = abBerMessage[wIndex++];
               // length of rest
               wIndex = wIndex + get_length_length(&abBerMessage[wIndex]);
               // Request ID
               copy_SNMP_request_number(&abBerMessage[wIndex],ptSNMPMessage -> tPUDDataHead.bRequestNumber,&wIndex);
               // Error status
               ptSNMPMessage -> tPUDDataHead.wErrorStatus = get_SNMP_integer(&abBerMessage[wIndex],&wIndex);
               // Error number
               ptSNMPMessage -> tPUDDataHead.wErrorNumber = get_SNMP_integer(&abBerMessage[wIndex],&wIndex);
               if (abBerMessage[wIndex++] == SNMP_SEQUENCE)
               { // Message length
                   wIndex = wIndex + get_length_length(&abBerMessage[wIndex]);
                   // Get bindings
                   ptSNMPMessage -> tPUDDataHead.wNumberOfBindings = 0;
                   while (wIndex < wLength)
                   {
                       if (abBerMessage[wIndex++] == SNMP_SEQUENCE)
                       { // Single Messages
                           wIndex = wIndex + get_length_length(&abBerMessage[wIndex]);
                           if ((iReturn = get_OID_binding(&abBerMessage[wIndex],
                                                           ptSNMPMessage -> tPUDBindings[ptSNMPMessage -> tPUDDataHead.wNumberOfBindings].abOIDBinding,
                                                           &wIndex))==NO_ERR)
                           {
                               if ((iReturn = get_binding_value(&abBerMessage[wIndex],
                                                           ptSNMPMessage -> tPUDBindings[ptSNMPMessage -> tPUDDataHead.wNumberOfBindings].abValueBinding,
                                                           &wIndex))==NO_ERR)
                               {
                               }
                               else
                               {
                                   iReturn = CONFIG_ERR;
                                   break;
                               }
                           }
                           else
                           {
                               iReturn = CONFIG_ERR;
                               break;
                           }

                           ptSNMPMessage -> tPUDDataHead.wNumberOfBindings += 1; // 1 Binding
                           if (ptSNMPMessage -> tPUDDataHead.wNumberOfBindings > MAX_NUMBER_BINDINGS)
                           {
                               iReturn = SNMP_IN_TO_BIG_ERR;
                               break;
                           }
                       }
                       else
                       {
                           iReturn = CONFIG_ERR;
                           break;
                       }

                   } // End While
               }
               else
               {
                   iReturn = CONFIG_ERR;
               }
           }
           else
           {
               iReturn = SNMP_STRING_TO_LONG_ERROR;
           }
       }
       else
       {
           iReturn = SNMP_COMMUNITY_GET_ERROR;
       }
   }
   else
   {
       iReturn = CONFIG_ERR;
   }
   return iReturn;
}

/*******************************************************************************************/
// Convert SNMP message to BER (Basic Encoding Rules)
/*******************************************************************************************/
int SNMP_convert_to_BER(tSNMPMessage * ptSNMPMessage, unsigned char * abBerMessage)
{
   int iReturn = NO_ERR;
   unsigned int wLengthTotal;
   unsigned int wLengthAllBinding;
   unsigned int wLengthBinding[MAX_NUMBER_BINDINGS];
   unsigned int wLengthPDU;
   unsigned int wIndex ;
   unsigned int wCounter;


   if (ptSNMPMessage ->tPUDDataHead.wNumberOfBindings < MAX_NUMBER_BINDINGS)
   {
       for (wCounter = 0; wCounter < ptSNMPMessage ->tPUDDataHead.wNumberOfBindings; wCounter ++)
       {
           if (wCounter < MAX_NUMBER_BINDINGS)
           {
               wLengthBinding[wCounter] = 0;
               wLengthBinding[wCounter] += get_BER_length_with_length(&ptSNMPMessage ->tPUDBindings[wCounter].abOIDBinding[1]);
               wLengthBinding[wCounter] += get_BER_length_with_length(&ptSNMPMessage ->tPUDBindings[wCounter].abValueBinding[1]);
           }
       }
       // Sequence +  length of length
       wLengthAllBinding =0;
       for (wCounter = 0; wCounter < ptSNMPMessage ->tPUDDataHead.wNumberOfBindings; wCounter ++)
       {
           wLengthAllBinding += wLengthBinding[wCounter] + 1 + number_bytes_length(wLengthBinding[wCounter]);
       }
       wLengthPDU = wLengthAllBinding + number_bytes_length(wLengthAllBinding);
       wLengthPDU += 1; // Sequence
       wLengthPDU += 6; // Error status, Error index
       wLengthPDU += get_BER_length_with_length(&ptSNMPMessage -> tPUDDataHead.bRequestNumber[1]);

       wLengthTotal = wLengthPDU + number_bytes_length(wLengthPDU);
       wLengthTotal +=1; // PDU type
       wLengthTotal += LENGTH_SNMP_VERSION; // Length SNMP Version
       wLengthTotal += 2+ strlen((char *)ptSNMPMessage-> tSNMPHeadData.bCommunityString);

       wIndex =0;
       // Start of message
       abBerMessage[wIndex++] = SNMP_SEQUENCE;
       // Complete length
       insert_BER_length(&abBerMessage[wIndex],wLengthTotal,&wIndex);
       // SNMP version
       insert_BER_word(&abBerMessage[wIndex],ptSNMPMessage-> tSNMPHeadData.bVersion, &wIndex);
       // Community String
       insert_BER_string(&abBerMessage[wIndex],ptSNMPMessage-> tSNMPHeadData.bCommunityString, &wIndex);
       // SNMP type
       abBerMessage[wIndex++]=  ptSNMPMessage-> tPUDDataHead.bType;
       // Length of PDU
       insert_BER_length(&abBerMessage[wIndex],wLengthPDU,&wIndex);
       // SNMP number
       copy_SNMP_request_number(ptSNMPMessage-> tPUDDataHead.bRequestNumber,&abBerMessage[wIndex], &wIndex);
       // SNMP error
       insert_BER_word(&abBerMessage[wIndex],ptSNMPMessage-> tPUDDataHead.wErrorStatus, &wIndex);
       // SNMP error number
       insert_BER_word(&abBerMessage[wIndex],ptSNMPMessage-> tPUDDataHead.wErrorNumber, &wIndex);
       // All bindings
       abBerMessage[wIndex++]= SNMP_SEQUENCE;
       insert_BER_length(&abBerMessage[wIndex],wLengthAllBinding,&wIndex);
       for (wCounter = 0; wCounter < ptSNMPMessage ->tPUDDataHead.wNumberOfBindings; wCounter ++)
       {
           abBerMessage[wIndex++]= SNMP_SEQUENCE;
           insert_BER_length(&abBerMessage[wIndex],wLengthBinding[wCounter],&wIndex);
           // SNMP oid
           insert_BER_oid(&abBerMessage[wIndex],ptSNMPMessage-> tPUDBindings[wCounter].abOIDBinding, &wIndex);
           // SNMP value
           insert_BER_value(&abBerMessage[wIndex],ptSNMPMessage-> tPUDBindings[wCounter].abValueBinding, &wIndex);
       }
   }// if (ptSNMPMessage ->tPUDDataHead.wNumberOfBindings < MAX_NUMBER_BINDINGS)
   else
   {
       iReturn = RANGE_ERR;
   }
   return iReturn;
}



/****************************************************************************************/
//Get length of BER message, return 0, if  length >  2 byte
/****************************************************************************************/
unsigned int get_BER_send_length(unsigned char * abBerLength, unsigned int *wIndex)
{
   unsigned int wReturn ;

   if (abBerLength[0] & 0x80)
   { //Bit 7 !=1 ==> more than one bytes
       if ((abBerLength[0] & 0x7F) == 2)
       {
           // Length + identifier + number bytes + 2 bytes
           wReturn = ((unsigned int)abBerLength[1]<<8) + (unsigned int) abBerLength[2] +  4;
           *wIndex = *wIndex + 3;
       }
       else
       {
           if ((abBerLength[0] & 0x7F) == 1)
           {
               // Length + Identifier + 1 Byte
               wReturn = (unsigned int)abBerLength[1]+  3;
               *wIndex = *wIndex + 2;
           }
           else
           {
               wReturn = 0;
           }
       }
   }//if (abBerLength[0] & 0x80)
   else
   {
       wReturn = (unsigned int)abBerLength[0]+2; // Length + identifier + 1 byte
       *wIndex = *wIndex+ 1;
   }
   return wReturn;
}

/**************************************************************************************/
// Return length of  BER message including "Length of length" and  message ID
/**************************************************************************************/
static unsigned int near get_BER_length_with_length(unsigned char * pbMessage)
{
   unsigned int wLength = 0xffff;

   if (pbMessage[0] & 0x80)   //>127
   {
       if ((pbMessage[0] & 0x7F) == 1)
       {   // 1 Databyte
           wLength = pbMessage[1] + 3;
       }
       else
       {
           if ((pbMessage[0] & 0x7F) == 2)
           {
               wLength = ((unsigned int)pbMessage[1]<<8) + pbMessage[2] + 4;
           }
           else
           {
               // invalid length, return 0xffff
           }
       }
   }//if (pbMessage[0] & 0x80)   //>127
   else
   {
       wLength = pbMessage[0]+2;
   }
   return wLength;
}

/**************************************************************************************/
// Get length of  "length"
/**************************************************************************************/
static unsigned int near get_length_length(unsigned char * pbMessage)
{
   unsigned int wLength = 0;

   if (pbMessage[0] & 0x80)
   { // ueber 127 hinaus
       wLength = (pbMessage[0] & 0x7F )+1;
   }
   else
   {
       wLength = 1;
   }
   return wLength;
}


/**************************************************************************************/
/*Insert length
/**************************************************************************************/
static int near insert_BER_length(unsigned char *pbBerMessage,unsigned int wLength,unsigned int *pbIndex)
{
   if (wLength > MIN_NUMBER_BER_1_BYTE)
   {
       *pbIndex +=  3;
       pbBerMessage[0] = 0x82;
       pbBerMessage[1] = (wLength>>8);
       pbBerMessage[2] = (unsigned char)wLength;
   }
   else
   {
       pbBerMessage[0] = wLength;
       *pbIndex += 1;
   }
   return NO_ERR;
}


/**************************************************************************************/
// Insert string
/**************************************************************************************/
int insert_BER_string(unsigned char *pbBerMessage, char *pbString, unsigned int *pwIndex)
{
   unsigned int wIndex = 0;
   int iReturn = NO_ERR;

   pbBerMessage[wIndex++]= SNMP_STRING;
   insert_BER_length(&pbBerMessage[wIndex],strlen((char *)pbString),&wIndex);
   memcpy(&pbBerMessage[wIndex],pbString,strlen((char *)pbString));
   *pwIndex += wIndex + strlen((char *)pbString);
   return iReturn;
}

/**************************************************************************************/
// Insert word
/**************************************************************************************/
int insert_BER_word(unsigned char *pbBerMessage, unsigned int wValue, unsigned int * wIndex)
{
   int iReturn = NO_ERR;

   pbBerMessage[0] = SNMP_INTEGER;
   if (wValue >= MIN_NUMBER_BER_1_BYTE)
   {
       pbBerMessage[3] = wValue;
       pbBerMessage[2] = (unsigned char)(wValue>>8);// Value
       pbBerMessage[1] = 2;       // Length
       *wIndex += 4;
   }
   else
   {
       pbBerMessage[2] = wValue; // Value
       pbBerMessage[1] = 1;     // Length
       *wIndex += 3;
   }
   return iReturn;
}

/**************************************************************************************/
// Insert OID
/**************************************************************************************/
static int near insert_BER_oid(unsigned char *pbBerMessage, unsigned char * pbAddress, unsigned int *pwIndex)
{
   int iReturn = NO_ERR;
   unsigned int wLength;
   unsigned int wCounter;

   if (pbAddress[0] == SNMP_OBJID)
   {
       if ( (!(pbAddress[1] & 0x80)) &&((pbAddress[1] & 0x7F)< MAX_OID_LENGTH)  )
       {
           wLength = (pbAddress[1] & 0x7F) + 2;
           for (wCounter = 0; wCounter < wLength; wCounter++)
           {
                   pbBerMessage[wCounter] = pbAddress[wCounter];
           }
           *pwIndex += wLength ;
       }
       else
       {
           iReturn = RANGE_ERR;
       }
   }
   else
   {
       iReturn = CONFIG_ERR;
   }
   return iReturn;
}

/**************************************************************************************/
// Insert value
/**************************************************************************************/
int insert_BER_value(unsigned char * pbDest , unsigned char *pbSource, unsigned int *pwIndex)
{
   int iReturn = NO_ERR;
   unsigned int wLength;
   unsigned int wCounter;

   wLength = get_BER_length_with_length(&pbSource[1]);
   for (wCounter = 0; wCounter < wLength; wCounter++)
   {
       pbDest[wCounter] = pbSource[wCounter];
       *pwIndex +=1;
   }
   return iReturn;
}

/**************************************************************************************/
// Convert OID to string
/**************************************************************************************/
int convert_OID_to_string(char * pbString, unsigned char * pbOID)
{
   int iReturn= NO_ERR;
   unsigned int wLength;
   unsigned int wCounter;
   char abTemp[6];

   pbString[0] = 0;
   if (pbOID[0] == SNMP_OBJID )
   {
       if (!(pbOID[1] & 0x80))
       {
           wLength = pbOID[1];
           for (wCounter = 0; wCounter < wLength; wCounter++)
           {
               if ((wCounter == 0) && (pbOID[wCounter+2] == 0x2b))
               {
                   strcpy(pbString,"1.3");
               }
               else
               {
                   if (wCounter == 0)
                   {
                       if (pbOID[wCounter+2] == 0x28)
                       {
                           strcpy(pbString,"1");
                       }
                       else
                       {
                           strcat(pbString, itoa((unsigned int)pbOID[wCounter+2],abTemp,10));
                       }
                   }
                   else
                   {

                       strcat(pbString,".");
                       strcat(pbString, itoa((unsigned int)pbOID[wCounter+2],abTemp,10));
                   }
               }
           }
       }
       else
       {
           iReturn = RANGE_ERR;
       }
   }
   else
   {
       iReturn = CONFIG_ERR;
   }
   return iReturn;

}
/**************************************************************************************/
// Return string, max. length 127 bytes
/**************************************************************************************/
unsigned char *  get_SNMP_string(unsigned char * pbMessage, unsigned int *wStringLength, unsigned int *wIndex )
{
   unsigned char * pbReturn;


   if (pbMessage[0] == SNMP_STRING)
   {
       if (pbMessage[1] & 0x80)
       {   // invalid length
           pbReturn = NULL;
           *wIndex = *wIndex + get_BER_length_with_length(&pbMessage[1]);
       }
       else
       {
           *wIndex += pbMessage[1]+2;  // + Length + 1 Byte Length + 1 Byte command
           *wStringLength = pbMessage[1];
           pbReturn = &pbMessage[2];  // String
       }
   }
   else
   {
       pbReturn = NULL;
   }
   return pbReturn;
}



/**************************************************************************************/
// Return IntegerValue (max. 16 Bit)
/**************************************************************************************/
int get_SNMP_integer(unsigned char * pbMessage, unsigned int * wIndex)
{
   int iReturn = 0;
   unsigned char bLength;

   if (pbMessage[0] == SNMP_INTEGER)
   {
       bLength = pbMessage[1];
       if (bLength < 128)
       {   // < 128
           if (bLength == 0x01)
           {
               iReturn = (char)pbMessage[2];
           }
           else
           {
               if (bLength >= 0x02)
               {
                   iReturn = ((unsigned int)pbMessage[bLength+1]) + ((unsigned int)pbMessage[bLength]<<8);
               }
           }
           *wIndex = *wIndex + bLength+2;
       }
   }
   return iReturn;
}

/**************************************************************************************/
//Return version number of SNMP message
/**************************************************************************************/
unsigned char get_SNMP_version(unsigned char * pbMessage, unsigned int * wIndex)
{
   unsigned char bReturn = 0 ;

   if (pbMessage[0] == SNMP_INTEGER)
   {
       bReturn = get_SNMP_integer(pbMessage, wIndex);
   }
   return bReturn;
}

/**************************************************************************************/
//Get OID of  binding
/**************************************************************************************/
int get_OID_binding(unsigned char * pbMessage, unsigned char * pbOIDString, unsigned int * pwIndex )
{
   int iReturn = NO_ERR;
   unsigned int wLength;
   unsigned int wCounter;

   if (pbMessage[0] == SNMP_OBJID)
   {
       wLength = get_BER_length_with_length(&pbMessage[1]);
       if (wLength < MAX_OID_LENGTH)
       {
           *pwIndex += wLength;
           for (wCounter = 0; wCounter < wLength; wCounter ++)
           {
               pbOIDString[wCounter] = pbMessage[wCounter];
           }
       }
       else
       {
           iReturn = RANGE_ERR;
       }
   }
   else
   {
       iReturn = CONFIG_ERR;
   }
   return iReturn;
}

/**************************************************************************************/
//Get value of binding
/**************************************************************************************/
int get_binding_value(unsigned char * pbMessage, unsigned char * pbValue, unsigned int *  pwIndex)
{
   int iReturn = NO_ERR;
   unsigned int wLength;
   unsigned int wCounter;

   wLength = get_BER_length_with_length(&pbMessage[1]);
   if (wLength < MAX_OID_VALUE_LENGTH)
   {
       *pwIndex += wLength;
       for (wCounter = 0; wCounter < wLength; wCounter ++)
       {
           pbValue[wCounter] = pbMessage[wCounter];
       }
   }
   else
   {
       iReturn = RANGE_ERR;
   }

   return iReturn;
}

/**************************************************************************************/
//Convert string to OID
/**************************************************************************************/
int string_to_oid(unsigned char * pbBer,char * pbOIDString)
{
   int iReturn = NO_ERR;
   char abTempString[OID_ADRESS_STRING_LENGTH];
   char *pbString;
   unsigned int wIndex;

   strcpy(abTempString,pbOIDString);
   pbOIDString = &abTempString[0];
   pbString = &abTempString[0];

   wIndex = 3;

   while (*pbString != NULL)
   {
       if (*pbString == '.')
       {
           *pbString = NULL;
           pbBer[wIndex++] = atoi(pbOIDString);
           pbOIDString = pbString+1;
       }
       pbString++;
   }
   if (pbString > pbOIDString)
   {
       pbBer[wIndex++] = atoi(pbOIDString);
   }
   pbBer[2] = 0x2B;
   pbBer[0] = SNMP_OBJID;
   pbBer[1] = wIndex-2;

   return iReturn;
}


/**************************************************************************************/
//Convert BER message to string (if type == string)
/**************************************************************************************/
int  convert_BER_to_string(unsigned char * pbMessage, char *sString)
{
   int iReturn = NO_ERR;
   unsigned int wLength;
   unsigned int wIndex;


   sString[0] = 0;
   if (pbMessage[0] == SNMP_STRING)
   {
       if (pbMessage[1] & 0x80)
       { // String zu lang
           if  ((pbMessage[1] & 0x7F) == 1)
           {
               wIndex = 3;
               wLength= pbMessage[2];
           }
           else
           {
               if  ((pbMessage[1] & 0x7F) == 2)
               {
                   wIndex = 4;
                   wLength= ((unsigned int)pbMessage[2]<<8)+pbMessage[3];
               }
               else
               {
                   iReturn = RANGE_ERR;
               }
           }
           iReturn = CONFIG_ERR;
       }
       else
       {
           wLength = pbMessage[1];
           wIndex = 2;
       }
       if (wLength < (MAX_LENGTH_SNMP_STRING-1))
       {
           memcpy(sString,&pbMessage[wIndex],wLength);
           sString[wLength] = 0;
       }
       else
       {
           iReturn = RANGE_ERR;
       }

   }
   else
   {
       iReturn = CONFIG_ERR;
   }
   return iReturn;
}


/**************************************************************************************/
// Copy SNMP request number
/**************************************************************************************/
int copy_SNMP_request_number(unsigned char * abSource, unsigned char * abDest, unsigned int* pwIndex)
{
   int iReturn = NO_ERR;
   unsigned int wLength;

   wLength = get_BER_length_with_length(&abSource[1]);
   if (wLength < MAX_LEN_REQUEST)
   {
       memcpy(abDest,abSource,wLength);
       *pwIndex += wLength;
   }
   else
   {
       iReturn = RANGE_ERR;
       insert_BER_word(abSource,0,pwIndex);
   }
   return iReturn;
}


/**************************************************************************************/
// Insert dword
/**************************************************************************************/

int insert_BER_dword(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex)
{
   int iReturn = NO_ERR;
   char cValue;
   int iValue;

   pbBerMessage[0] = SNMP_INTEGER;
   if ((dwValue & 0xffffff80L)== 0xffffff80L)
   {
       cValue = (char)dwValue;
       pbBerMessage[1] = 1;       // Length
       pbBerMessage[2] = cValue;
   }
   else
   {
       if ((dwValue & 0xffff8000L)==0xffff8000L)
       {
           iValue = (int)dwValue;
           pbBerMessage[1] = 2;       // Length
           pbBerMessage[3] = iValue;
           pbBerMessage[2] = (unsigned char)(iValue>>8);
       }
       else
       {
          pbBerMessage[1] = 4;       // Length
          pbBerMessage[5] = (unsigned char)dwValue;
          pbBerMessage[4] = (unsigned char)(dwValue>>8);
          pbBerMessage[3] = (unsigned char)(dwValue>>16);
          pbBerMessage[2] = (unsigned char)(dwValue>>24);
       }
   }


   *wIndex += 6;
   return iReturn;
}

/**************************************************************************************/
// Get dword
/**************************************************************************************/

unsigned long get_SNMP_dword(unsigned char *pbMessage, unsigned int* wIndex)
{
   unsigned long dwReturn = 0;
   unsigned int wLength =0;
   unsigned int wIndexPosNumber = 0;

   if (pbMessage[0] == SNMP_INTEGER)
   {
       if (pbMessage[1] & 0x80)
       {
           if ((pbMessage[1] & 0x7F) == 1)
           {
               wLength = pbMessage[2] ;
               wIndexPosNumber = 3;
           }
           else
           {
               if ((pbMessage[1] & 0x7F) == 2)
               {
                   wLength = ((unsigned int)pbMessage[2]<<8) + pbMessage[3] ;
                   wIndexPosNumber = 4;
               }
           }
       }
       else
       {
           wLength = pbMessage[1];
           wIndexPosNumber = 2;
       }

       if (wLength == 0x01)
       {
           dwReturn = pbMessage[wIndexPosNumber];
       }
       else
       {
           if (wLength == 0x02)
           {
               dwReturn = ((unsigned long)pbMessage[wIndexPosNumber+1])
                           + ((unsigned long)pbMessage[wIndexPosNumber]<<8);
           }
           else
           {
               if (wLength == 0x03)
               {
                   dwReturn = ((unsigned long)pbMessage[wIndexPosNumber+2])
                               + ((unsigned long)pbMessage[wIndexPosNumber+1]<<8)
                               + ((unsigned long)pbMessage[wIndexPosNumber]<<16);
               }
               else
               {
                   if (wLength == 0x04)
                   {
                       dwReturn = ((unsigned long)pbMessage[wIndexPosNumber+3])
                                 +((unsigned long)pbMessage[wIndexPosNumber+2]<<8)
                                 +((unsigned long)pbMessage[wIndexPosNumber+1]<<16)
                                 + ((unsigned long)pbMessage[wIndexPosNumber]<<24);
                   }
               }
           }
       }
   }
   wIndex +=  get_BER_length_with_length(&pbMessage[1]);
   return dwReturn;
}


/**************************************************************************************/
// Insert timeticks
/**************************************************************************************/

int insert_BER_timeticks(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex)
{
   int iReturn = NO_ERR;

   pbBerMessage[0] = SNMP_TIMETICKS;
   pbBerMessage[1] = 4;       // Length
   pbBerMessage[5] = (unsigned char)dwValue;
   pbBerMessage[4] = (unsigned char)(dwValue>>8);
   pbBerMessage[3] = (unsigned char)(dwValue>>16);
   pbBerMessage[2] = (unsigned char)(dwValue>>24);

   *wIndex += 6;
   return iReturn;
}

/**************************************************************************************/
// Insert mac address
/**************************************************************************************/

int insert_BER_macaddress(unsigned char * pbBerMessage, unsigned char * pbAddress, unsigned int * wIndex)
{
   int iReturn = NO_ERR;

   pbBerMessage[0] = SNMP_STRING;
   pbBerMessage[1] = 6;       // Length
   pbBerMessage[2] = pbAddress[0];
   pbBerMessage[3] = pbAddress[1];
   pbBerMessage[4] = pbAddress[2];
   pbBerMessage[5] = pbAddress[3];
   pbBerMessage[6] = pbAddress[4];
   pbBerMessage[7] = pbAddress[5];
   *wIndex += 8;
   return iReturn;
}




/**************************************************************************************/
// Insert IP
/**************************************************************************************/

int insert_BER_ipaddress(unsigned char * pbBerMessage, unsigned char * pbAddress, unsigned int * wIndex)
{
   int iReturn = NO_ERR;

   pbBerMessage[0] = SNMP_IPADDRESS;
   pbBerMessage[1] = 4;       // Length
   pbBerMessage[2] = pbAddress[0];
   pbBerMessage[3] = pbAddress[1];
   pbBerMessage[4] = pbAddress[2];
   pbBerMessage[5] = pbAddress[3];
   *wIndex += 6;
   return iReturn;
}

/**************************************************************************************/
// Insert counter
/**************************************************************************************/

int insert_BER_counter(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex)
{
   int iReturn = NO_ERR;

   pbBerMessage[0] = SNMP_COUNTER;
   pbBerMessage[1] = 4;       // Length
   pbBerMessage[5] = (unsigned char)dwValue;
   pbBerMessage[4] = (unsigned char)(dwValue>>8);
   pbBerMessage[3] = (unsigned char)(dwValue>>16);
   pbBerMessage[2] = (unsigned char)(dwValue>>24);

   *wIndex += 6;
   return iReturn;
}

/**************************************************************************************/
// Insert gauge
/**************************************************************************************/

int insert_BER_gauge(unsigned char *pbBerMessage, unsigned long dwValue, unsigned int * wIndex)
{
   int iReturn = NO_ERR;

   pbBerMessage[0] = SNMP_GAUGE;
   pbBerMessage[1] = 4;
   pbBerMessage[5] = (unsigned char)dwValue;
   pbBerMessage[4] = (unsigned char)(dwValue>>8);
   pbBerMessage[3] = (unsigned char)(dwValue>>16);
   pbBerMessage[2] = (unsigned char)(dwValue>>24);

   *wIndex += 6;
   return iReturn;
}

/**************************************************************************************/
// convert SNMP trap to BER
/**************************************************************************************/
int convert_SNMPtrap_to_BER(tSNMPTrap * ptTrapMessage, unsigned char *abBerString)
{
   int iReturn = NO_ERR;
   unsigned int wLengthTotal;
   unsigned int wLengthAllBinding;
   unsigned int wLengthBinding[MAX_NUMBER_TRAP_BINDINGS];
   unsigned int wLengthPDU;
   unsigned int wIndex ;
   unsigned int wCounter;


   if (ptTrapMessage ->tTrapDataHead.wNumberOfBindings < MAX_NUMBER_TRAP_BINDINGS)
   {
       for (wCounter = 0; wCounter < ptTrapMessage ->tTrapDataHead.wNumberOfBindings; wCounter ++)
       {
           if (wCounter <  MAX_NUMBER_TRAP_BINDINGS)
           {
               wLengthBinding[wCounter] = 0;
               wLengthBinding[wCounter] += get_BER_length_with_length(&ptTrapMessage ->tPUDBindings[wCounter].abOIDBinding[1]);
               wLengthBinding[wCounter] += get_BER_length_with_length(&ptTrapMessage ->tPUDBindings[wCounter].abValueBinding[1]);
           }
       }


       // Sequence + "Length of Length"
       wLengthAllBinding =0;
       for (wCounter = 0; wCounter < ptTrapMessage ->tTrapDataHead.wNumberOfBindings; wCounter ++)
       {
           wLengthAllBinding += wLengthBinding[wCounter] + 1 + number_bytes_length(wLengthBinding[wCounter]);
       }
       wLengthPDU = wLengthAllBinding+ number_bytes_length(wLengthAllBinding);
       wLengthPDU += 1; // sequence
       wLengthPDU += 6; // Time Ticks
       // Enterprise MIB
       wLengthPDU += get_BER_length_with_length(&ptTrapMessage -> tTrapDataHead.bEnterpriseMib[1]);
       wLengthPDU += 6; // Secific Trap Code, Trap Code
       wLengthPDU += 6; // IP address

       wLengthTotal = wLengthPDU + number_bytes_length(wLengthPDU);
       wLengthTotal +=1; // PDU type
       wLengthTotal += LENGTH_SNMP_VERSION; // Length SNMP Version
       wLengthTotal += 2+ strlen((char *)ptTrapMessage-> tSNMPHeadData.bCommunityString);

       wIndex =0;
       // Start of message
       abBerString[wIndex++] = SNMP_SEQUENCE;
       // Length of message
       insert_BER_length(&abBerString[wIndex],wLengthTotal,&wIndex);
       // SNMP version
       insert_BER_word(&abBerString[wIndex],ptTrapMessage-> tSNMPHeadData.bVersion, &wIndex);
       // Community string
       insert_BER_string(&abBerString[wIndex],ptTrapMessage-> tSNMPHeadData.bCommunityString, &wIndex);
       // SNMP type
       abBerString[wIndex++]=  ptTrapMessage-> tTrapDataHead.bPDUType;
       // Length der PDU
       insert_BER_length(&abBerString[wIndex],wLengthPDU,&wIndex);
       // Enterprise Mib
       insert_BER_oid(&abBerString[wIndex],ptTrapMessage-> tTrapDataHead.bEnterpriseMib, &wIndex);
       // Source IP Address
       insert_BER_ipaddress(&abBerString[wIndex],ptTrapMessage-> tTrapDataHead.bAgentIPAddress ,&wIndex);
       // SNMP Trap type
       insert_BER_word(&abBerString[wIndex],ptTrapMessage-> tTrapDataHead.wStandardTrapType, &wIndex);
       // SNMP Trap type
       insert_BER_word(&abBerString[wIndex],ptTrapMessage-> tTrapDataHead.wSpecificTrapType, &wIndex);
       // Time stamp
       insert_BER_timeticks(&abBerString[wIndex],ptTrapMessage-> tTrapDataHead.dwTimeStamp ,&wIndex);
       // All bindings
       abBerString[wIndex++]= SNMP_SEQUENCE;
       insert_BER_length(&abBerString[wIndex],wLengthAllBinding,&wIndex);
       for (wCounter = 0; wCounter < ptTrapMessage ->tTrapDataHead.wNumberOfBindings; wCounter ++)
       {
           abBerString[wIndex++]= SNMP_SEQUENCE;
           insert_BER_length(&abBerString[wIndex],wLengthBinding[wCounter],&wIndex);
           // SNMP OID
           insert_BER_oid(&abBerString[wIndex],ptTrapMessage-> tPUDBindings[wCounter].abOIDBinding, &wIndex);
           // SNMP value
           insert_BER_value(&abBerString[wIndex],ptTrapMessage-> tPUDBindings[wCounter].abValueBinding, &wIndex);
       }

   }
   else
   {
       iReturn = RANGE_ERR;
   }

   return iReturn;
}



static unsigned char near number_bytes_length(unsigned int wLength)
{
   unsigned char bReturn;

   if (wLength < MIN_NUMBER_BER_1_BYTE)
   {
       bReturn = 1;
   }
   else
   {
       bReturn = 3;
   }
   return bReturn;
}


void append_ip_to_oidstring(unsigned long ip,char *destOidStr)
{
   char abTemp[17];
   char abTempS[6];
   unsigned char abIP_AddressTemp[4];
   unsigned char h;


   ip_to_array(ip,abIP_AddressTemp); //append local ip address
   for (h = 0; h < 4; h++)
   {
      strcat(destOidStr,".");
      if (abIP_AddressTemp[h] & 0x80)
      {
        strcpy(abTemp,"129.");
        strcat(abTemp,itoa((unsigned int) abIP_AddressTemp[h]-128,abTempS,10));
      }
      else
      {
          strcpy(abTemp,itoa((unsigned int) abIP_AddressTemp[h],abTempS,10));
      }
      strcat(destOidStr,abTemp);
   }
}




/*
   Integer values are represented using two complement notation
   This notation can be briefly summarized as follows.
   Consider an integer in two's complement representation, stored in
   an N-bit field. there are two cases: nonnegative and negative

   Nonnegative numbers are represented as follows. The leftmost (MSB) bit
   is zero and the remaining bits are the binary form of the magnitude of the number.
   A negative number whose magnitude is X and for which X has a value of less than or equal 2^n-1
   is represented by 2^N   - X. For all negative numbers, the MSB bit has value 1.
   Thus this bit functions as a sign bit.
*/

void append_port_to_oidstring(unsigned int port,char *destOidStr)
{
   char abTempS[6];
   char abTemp[17];
   int h;
   unsigned int wIndex=0;
   unsigned char pbBerMessageTemp[4];

   insert_BER_word(pbBerMessageTemp, port, &wIndex);
   for(h=2;h<wIndex;h++)
   {
     if (pbBerMessageTemp[h] & 0x80)
     {
           strcpy(abTemp,"129.");
           strcat(abTemp,itoa((unsigned int) pbBerMessageTemp[h]-128,abTempS,10));
     }
     else
     {
        if((pbBerMessageTemp[h]) || (wIndex==3))
           strcpy(abTemp,itoa((unsigned int) pbBerMessageTemp[h],abTempS,10));
        else
           abTemp[0]=0;
     }
     if(abTemp[0])
     {
       strcat(destOidStr,abTemp);
       strcat(destOidStr,".");
     }
   }
   if(destOidStr[strlen(destOidStr)-1]=='.')
       destOidStr[strlen(destOidStr)-1]=0;
}

void ip_to_array(unsigned long ip, unsigned char * bytearray)
{

    bytearray[3] = (unsigned char)(ip >>24);
    bytearray[2] = (unsigned char)(ip >>16);
    bytearray[1] = (unsigned char)(ip >> 8);
    bytearray[0] = (unsigned char)(ip);
}
/**************************************************************************************************************/
//end snmphlp.c
/**************************************************************************************************************/
