/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     timerin.c
* Function:   Example of using the hardware api with timer inputs.
*
*******************************************************************************
*
* $Header: TIMERIN.C, 2, 31.03.2004 11:16:05, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "clib.h"

/******************************************************************************
* Variables
******************************************************************************/
void interrupt (*old_timer1C_handler)();
volatile unsigned long int_counter;
volatile unsigned long hz;
unsigned char timer;

/******************************************************************************
* Interrupt handler
******************************************************************************/
void huge int_handler (void)
{
  int_counter++;
}

/******************************************************************************
* Timer handler
******************************************************************************/
void interrupt timer1C_handler (void)
{
  unsigned counter;

  // Read timer count
  counter = hal_read_timer_count(timer, NULL);

  // Reset timer count
  hal_write_timer_count(timer, 0);

  hz = (unsigned long) (65535L * int_counter) + counter;
  int_counter = 0;
}

/******************************************************************************
* Display command line options
******************************************************************************/
void usage (void)
{
  printf("\nProgram enables TMRIN0 or TMRIN1 pin and shows frequency");
  printf("\nUsage: timerin <timer>,  timer=0..1");
}

/***************************************************************************/
// Main
/***************************************************************************/
void main (int argc, char *argv[])
{
   unsigned      i;

   if (argc==2)
   {
      timer = atoi(argv[1]);
      if (timer>1)
      {
        usage();
        return;
      }
   }
   else
   {
     usage();
     return;
   }


   printf("Installing TMRIN%d ...", timer);
   int_counter =
   hz          = 0;

   // Enable timers
   if (timer==0)
   {
     pfe_enable_timer (0x0002); // enable TMRIN0
   }
   else
   {
     pfe_enable_timer (0x0008); // enable TMRIN1
   }

   // Set service interrupt routine
   hal_install_isr(8+timer, 1, int_handler);

   // Install 1Ch handler and set to 1s intervall
   BIOS_Set_Timer_0x1C (1000) ;
   old_timer1C_handler = getvect(0x1C);
   setvect(0x1C, timer1C_handler);

   // Init timer
   // continous, enable interrupt, ext clock
   // MaxCount count from 0000h-ffffh
   hal_init_timer(timer, 0x7, 0);

   // Start timer
   hal_start_timer(timer);

   // wait 1 sec
   RTX_Sleep_Time(1000);

   for (i=0; i<60; i++)
   {
     // wait 1 sec
     RTX_Sleep_Time(1000);

     printf("\nTimer%d = %lu Hz", timer, hz);
   }

   // Deinstall timer interrupt handler
   hal_install_isr(8+timer, 0, NULL);

   // Set old 1Ch handler
   setvect(0x1C, old_timer1C_handler);

   printf("\ndone");
}

