/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     udpcfgcb.c
* Function:   This program demonstrates the usage of the TCP/IP-API of the IPC@Chip.
*             It installs a user callback function on the UDP config server. This
*             function is called when the config server receives a user defined
*             message. The callback function prints the received message to STDOUT
*             and send an answer to the sender.
* Notes:      Use the windows program udpcfgcb to send user defined messages to
*             the config server.
*
*******************************************************************************
*
* $Header: udpcfgcb.c, 2, 31.03.2004 11:16:07, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include "clib.h"



/******************************************************************************
* Global variables
******************************************************************************/
UDPCFGSRV_USERCBINFO UdpCfgSrv_UserCBInfo;
char     buffer[512];



/******************************************************************************
* MyUdpCfgCB()
*
* Callback function
******************************************************************************/
void huge MyUdpCfgCB( struct UdpCfgSrv_UserCBInfo *info )
{
   char ipStr[16];

   if (info->length<sizeof(struct UdpCfgSrv_UserCBInfo) )
   {
     printf("\r\nUdp Config Info Struct invalid!");
     return;
   }

   memcpy( buffer, info->dataPtr, info->dataLength );
   buffer[info->dataLength] = '\0';
   InetToAscii(&info->fromAddrPtr->sin_addr.s_addr, ipStr );
   printf("\r\nReceived: <%s> from %s", info->dataPtr, ipStr );

   sprintf( buffer, "Received: <%s>", info->dataPtr );
   info->dataPtr    = buffer;
   info->dataLength = strlen(buffer);
   return;
}



/******************************************************************************
* main()
******************************************************************************/
void main(void)
{
   // Set STDIO focus to APPLICATION
   BIOS_Set_Focus( FOCUS_APPLICATION );

   printf("\r\nUdp Config Server Callback function test Program"
          "\r\nInstall Udp Cfg Srv CB function using Int A0h function 40h");

   // install Callback
   BIOS_Install_UDP_Cfg_Callback( MyUdpCfgCB );
   
   printf("\r\nFunction installed"
          "\r\nPress any key to exit.");
   
   while( !kbhit() )
   {
      // Sleep one second
      RTX_Sleep_Time( 1000 );
   }

   printf("\r\nRemoving Udp Cfg Srv CB");

   // remove Callback
   BIOS_Install_UDP_Cfg_Callback( NULL );

   printf("\r\nDone.");

   // Set STDIO focus to BOTH
   BIOS_Set_Focus( FOCUS_BOTH );

   return;
}
